@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,14 +11,34 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for querying account data.
 
+Functions
+---------
+download_account_balances
+    Query account balances.
+
+download_account_portfolio
+    Download account portfolio.
+
+download_exchange_rates
+    Query exchange rates for the base currency.
+
+Notes
+-----
+Usage Guide:
+
+* Account Monitoring: https://qrok.it/dl/qr/account
+"""
 import sys
 import requests
+from quantrocket.utils._typing import FilepathOrBuffer, Union, Literal
 from quantrocket.houston import houston
 from quantrocket.exceptions import NoAccountData
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
 
 __all__ = [
     "download_account_balances",
@@ -26,10 +46,17 @@ __all__ = [
     "download_exchange_rates",
 ]
 
-def download_account_balances(filepath_or_buffer=None, output="csv",
-                              start_date=None, end_date=None,
-                              latest=False, accounts=None, below=None,
-                              fields=None, force_refresh=False):
+def download_account_balances(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    start_date: str = None,
+    end_date: str = None,
+    latest: bool = False,
+    accounts: Union[list[str], str] = None,
+    below: dict[str, float] = None,
+    fields: Union[list[str], str] = None,
+    force_refresh: bool = False
+    ) -> None:
     """
     Query account balances.
 
@@ -70,6 +97,12 @@ def download_account_balances(filepath_or_buffer=None, output="csv",
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Account Monitoring: https://qrok.it/dl/qr/account
+
     Examples
     --------
     Query latest balances. You can use StringIO to load the CSV into pandas.
@@ -123,10 +156,18 @@ def _cli_download_account_balances(*args, **kwargs):
         kwargs["below"] = dict_strs_to_dict(*below)
     return json_to_cli(download_account_balances, *args, **kwargs)
 
-def download_account_portfolio(filepath_or_buffer=None, output="csv",
-                               brokers=None, accounts=None, sec_types=None,
-                               exchanges=None, sids=None, symbols=None,
-                               include_zero=False, fields=None):
+def download_account_portfolio(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    brokers: Union[list[Literal["alpaca", "ibkr"]], Literal["alpaca", "ibkr"]] = None,
+    accounts: Union[list[str], str] = None,
+    sec_types: Union[list[str], str] = None,
+    exchanges: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    symbols: Union[list[str], str] = None,
+    include_zero: bool = False,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Download current portfolio.
 
@@ -169,6 +210,12 @@ def download_account_portfolio(filepath_or_buffer=None, output="csv",
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Account Monitoring: https://qrok.it/dl/qr/account
+
     Examples
     --------
     Download current portfolio. You can use StringIO to load the CSV into pandas.
@@ -215,9 +262,15 @@ def download_account_portfolio(filepath_or_buffer=None, output="csv",
 def _cli_download_account_portfolio(*args, **kwargs):
     return json_to_cli(download_account_portfolio, *args, **kwargs)
 
-def download_exchange_rates(filepath_or_buffer=None, output="csv",
-                            start_date=None, end_date=None, latest=False,
-                            base_currencies=None, quote_currencies=None):
+def download_exchange_rates(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    start_date: str = None,
+    end_date: str = None,
+    latest: bool = False,
+    base_currencies: Union[list[str], str] = None,
+    quote_currencies: Union[list[str], str] = None
+    ) -> None:
     """
     Query exchange rates for the base currency.
 
@@ -252,6 +305,12 @@ def download_exchange_rates(filepath_or_buffer=None, output="csv",
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Account Monitoring: https://qrok.it/dl/qr/account
+
     Examples
     --------
     Query latest exchange rates. You can use StringIO to load the CSV into pandas.
