@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,16 +11,65 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for placing and managing orders and tracking positions and
+executions.
 
+Functions
+---------
+place_orders
+    Place one or more orders.
+
+cancel_orders
+    Cancel one or more orders by order ID, sid, or order ref.
+
+download_order_statuses
+    Download order statuses.
+
+download_positions
+    Query current positions and write results to file.
+
+list_positions
+    Query current positions and return them as a Python list.
+
+close_positions
+    Generate orders to close positions.
+
+download_executions
+    Query executions from the executions database.
+
+record_executions
+    Record executions that happened outside of QuantRocket's knowledge.
+
+apply_split
+    Apply a stock split to an open position.
+
+download_pnl
+    Query trading performance and return a CSV of results or PDF tearsheet.
+
+read_pnl_csv
+    Load a PNL CSV into a DataFrame.
+
+Notes
+-----
+Usage Guide:
+
+* Orders and Positions: https://qrok.it/dl/qr/orders
+* Performance Tracking: https://qrok.it/dl/qr/performance
+"""
 import sys
 import six
 import json
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import pandas as pd
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.stream import to_bytes
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.utils.parse import _read_moonshot_or_pnl_csv
+from quantrocket.utils._typing import FilepathOrBuffer, Union, Literal
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.stream import to_bytes
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket.utils._parse import _read_moonshot_or_pnl_csv
 
 __all__ = [
     "place_orders",
@@ -36,7 +85,10 @@ __all__ = [
     "read_pnl_csv",
 ]
 
-def place_orders(orders=None, infilepath_or_buffer=None):
+def place_orders(
+    orders: list[dict[str, Union[str, float]]] = None,
+    infilepath_or_buffer: FilepathOrBuffer = None
+    ) -> list[str]:
     """
     Place one or more orders.
 
@@ -58,6 +110,12 @@ def place_orders(orders=None, infilepath_or_buffer=None):
     list
         order IDs
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
+
     Examples
     --------
     >>> orders = []
@@ -108,8 +166,13 @@ def _cli_place_orders(*args, **kwargs):
         kwargs["orders"] = orders
     return json_to_cli(place_orders, *args, **kwargs)
 
-def cancel_orders(order_ids=None, sids=None, order_refs=None, accounts=None,
-                  cancel_all=None):
+def cancel_orders(
+    order_ids: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    cancel_all: bool = None
+    ) -> dict[str, str]:
     """
     Cancel one or more orders by order ID, sid, or order ref.
 
@@ -135,6 +198,12 @@ def cancel_orders(order_ids=None, sids=None, order_refs=None, accounts=None,
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
+
     Examples
     --------
     Cancel orders by order ID:
@@ -172,10 +241,18 @@ def cancel_orders(order_ids=None, sids=None, order_refs=None, accounts=None,
 def _cli_cancel_orders(*args, **kwargs):
     return json_to_cli(cancel_orders, *args, **kwargs)
 
-def download_order_statuses(filepath_or_buffer=None, output="csv",
-                            order_ids=None, sids=None, order_refs=None,
-                            accounts=None, open_orders=None,
-                            start_date=None, end_date=None, fields=None):
+def download_order_statuses(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    order_ids: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    open_orders: bool = None,
+    start_date: str = None,
+    end_date: str = None,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Download order statuses.
 
@@ -216,6 +293,12 @@ def download_order_statuses(filepath_or_buffer=None, output="csv",
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
+
     Examples
     --------
     Download order status by order ID and load into Pandas:
@@ -274,9 +357,15 @@ def download_order_statuses(filepath_or_buffer=None, output="csv",
 def _cli_download_order_statuses(*args, **kwargs):
     return json_to_cli(download_order_statuses, *args, **kwargs)
 
-def download_positions(filepath_or_buffer=None, output="csv",
-                       order_refs=None, accounts=None, sids=None,
-                       view="blotter", diff=False):
+def download_positions(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    view: str = "blotter",
+    diff: bool = False
+    ) -> None:
     """
     Query current positions and write results to file.
 
@@ -323,6 +412,12 @@ def download_positions(filepath_or_buffer=None, output="csv",
     See Also
     --------
     list_positions : load positions into Python list
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
     """
     params = {}
     if order_refs:
@@ -356,8 +451,13 @@ def download_positions(filepath_or_buffer=None, output="csv",
 def _cli_download_positions(*args, **kwargs):
     return json_to_cli(download_positions, *args, **kwargs)
 
-def list_positions(order_refs=None, accounts=None, sids=None,
-                   view="blotter", diff=False):
+def list_positions(
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    view: str = "blotter",
+    diff: bool = False
+    ) -> list[dict[str, Union[str, float]]]:
     """
     Query current positions and return them as a Python list.
 
@@ -393,6 +493,12 @@ def list_positions(order_refs=None, accounts=None, sids=None,
     -------
     list
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
+
     Examples
     --------
     Query current positions and load into Pandas:
@@ -412,9 +518,14 @@ def list_positions(order_refs=None, accounts=None, sids=None,
     else:
         return []
 
-def close_positions(filepath_or_buffer=None, output="csv",
-                    order_refs=None, accounts=None, sids=None,
-                    params=None):
+def close_positions(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    params: dict[str, Union[str, float]] = None
+    ) -> None:
     """
     Generate orders to close positions.
 
@@ -450,6 +561,17 @@ def close_positions(filepath_or_buffer=None, output="csv",
     -------
     None
 
+    See Also
+    --------
+    place_orders : place one or more orders
+    record_executions : record executions that happened outside of QuantRocket’s knowledge
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Orders and Positions: https://qrok.it/dl/qr/orders
+
     Examples
     --------
     Get orders to close positions, then place the orders:
@@ -467,11 +589,6 @@ def close_positions(filepath_or_buffer=None, output="csv",
     >>> executions_file = io.StringIO()
     >>> close_positions(executions_file, sids="FIBBG123456", params={"Price": 23.50})
     >>> record_executions(infilepath_or_buffer=executions_file)
-
-    See Also
-    --------
-    place_orders : place one or more orders
-    record_executions : record executions that happened outside of QuantRocket’s knowledge
     """
     _params = {}
     if order_refs:
@@ -506,9 +623,14 @@ def _cli_close_positions(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(close_positions, *args, **kwargs)
 
-def download_executions(filepath_or_buffer=None,
-                        order_refs=None, accounts=None, sids=None,
-                        start_date=None, end_date=None):
+def download_executions(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    start_date: str = None,
+    end_date: str = None
+    ) -> None:
     """
     Query executions from the executions database.
 
@@ -535,6 +657,12 @@ def download_executions(filepath_or_buffer=None,
     Returns
     -------
     None
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Performance Tracking: https://qrok.it/dl/qr/performance
     """
     params = {}
     if order_refs:
@@ -559,7 +687,10 @@ def download_executions(filepath_or_buffer=None,
 def _cli_download_executions(*args, **kwargs):
     return json_to_cli(download_executions, *args, **kwargs)
 
-def record_executions(executions=None, infilepath_or_buffer=None):
+def record_executions(
+    executions: list[dict[str, Union[str, float]]] = None,
+    infilepath_or_buffer: FilepathOrBuffer = None
+    ) -> list[str]:
     """
     Record executions that happened outside of QuantRocket's knowledge.
 
@@ -599,6 +730,17 @@ def record_executions(executions=None, infilepath_or_buffer=None):
         a list of execution IDs generated by the blotter and inserted in the
         database
 
+    See Also
+    --------
+    close_positions : generate orders to close positions, or generate executions
+      to mark positions as closed
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Delisted Positions: https://qrok.it/dl/qr/delisted-positions
+
     Examples
     --------
     >>> executions = []
@@ -612,11 +754,6 @@ def record_executions(executions=None, infilepath_or_buffer=None):
         }
     >>> executions.append(execution1)
     >>> execution_ids = record_executions(executions)
-
-    See Also
-    --------
-    close_positions : generate orders to close positions, or generate executions
-      to mark positions as closed
     """
     if executions and infilepath_or_buffer:
         raise ValueError("executions and infilepath_or_buffer are mutually exclusive")
@@ -652,7 +789,11 @@ def _cli_record_executions(*args, **kwargs):
         kwargs["executions"] = executions
     return json_to_cli(record_executions, *args, **kwargs)
 
-def apply_split(sid, old_shares, new_shares):
+def apply_split(
+    sid: str,
+    old_shares: int,
+    new_shares: int
+    ) -> list[dict[str, Union[str, int]]]:
     """
     Apply a stock split to an open position.
 
@@ -682,6 +823,12 @@ def apply_split(sid, old_shares, new_shares):
     list
         the old and new position for this sid, by account and order ref
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Stock Splits: https://qrok.it/dl/qr//dl/qr/apply-split
+
     Examples
     --------
     Record a 2-for-1 split:
@@ -704,10 +851,17 @@ def apply_split(sid, old_shares, new_shares):
 def _cli_apply_split(*args, **kwargs):
     return json_to_cli(apply_split, *args, **kwargs)
 
-def download_pnl(filepath_or_buffer=None,
-                 order_refs=None, accounts=None, sids=None,
-                 start_date=None, end_date=None, timezone=None,
-                 details=False, output="csv"):
+def download_pnl(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    order_refs: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    start_date: str = None,
+    end_date: str = None,
+    timezone: str = None,
+    details: bool = False,
+    output: Literal["csv", "pdf"] = "csv"
+    ) -> None:
     """
     Query trading performance and return a CSV of results or PDF tearsheet.
 
@@ -748,6 +902,12 @@ def download_pnl(filepath_or_buffer=None,
     Returns
     -------
     None
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Performance Tracking: https://qrok.it/dl/qr/performance
     """
     params = {}
     if order_refs:
@@ -781,7 +941,9 @@ def download_pnl(filepath_or_buffer=None,
 def _cli_download_pnl(*args, **kwargs):
     return json_to_cli(download_pnl, *args, **kwargs)
 
-def read_pnl_csv(filepath_or_buffer):
+def read_pnl_csv(
+    filepath_or_buffer: FilepathOrBuffer
+    ) -> 'pd.DataFrame':
     """
     Load a PNL CSV into a DataFrame.
 
@@ -799,6 +961,12 @@ def read_pnl_csv(filepath_or_buffer):
         a multi-index (Field, Date[, Time]) DataFrame of backtest
         results, with sids or strategy codes as columns
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Performance Tracking: https://qrok.it/dl/qr/performance
+
     Examples
     --------
     >>> results = read_pnl_csv("pnl.csv")
