@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,15 +11,34 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for loading code.
 
+Functions
+---------
+clone
+    Clone files from a Git repository.
+
+Notes
+-----
+Usage Guide:
+
+* Code Management: https://qrok.it/dl/qr/codeload
+"""
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
 
 __all__ = [
     "clone",
 ]
 
-def clone(repo, branch=None, replace=None, skip_existing=None, target_dir=None):
+def clone(
+    repo: str,
+    branch: str = None,
+    replace: bool = None,
+    skip_existing: bool = None,
+    target_dir: str = None
+    ) -> dict[str, str]:
     """
     Clone files from a Git repository.
 
@@ -53,6 +72,12 @@ def clone(repo, branch=None, replace=None, skip_existing=None, target_dir=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Code Management: https://qrok.it/dl/qr/codeload
+
     Examples
     --------
     Clone QuantRocket's "umd" demo repository:
