@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,11 +11,57 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for managing QuantRocket databases.
 
+Functions
+---------
+list_databases
+    List databases.
+
+get_s3_config
+    Return the current S3 configuration, if any.
+
+set_s3_config
+    Set AWS S3 configuration for pushing and pulling databases to and from
+    S3.
+
+s3_push_databases
+    Push database(s) to Amazon S3.
+
+s3_pull_databases
+    Pull database(s) from Amazon S3.
+
+optimize_databases
+    Optimize databases to improve performance.
+
+connect_sqlite
+    Return a connection to a SQLite database.
+
+insert_or_fail
+    Insert a DataFrame into a SQLite database, failing on duplicates.
+
+insert_or_replace
+    Insert a DataFrame into a SQLite database, replacing duplicates.
+
+insert_or_ignore
+    Insert a DataFrame into a SQLite database, ignoring duplicates.
+
+Notes
+-----
+Usage Guide:
+
+* Database Management: https://qrok.it/dl/qr/db
+* Custom Data: https://qrok.it/dl/qr/custom-data
+"""
 import getpass
+from typing import TYPE_CHECKING, Union
+if TYPE_CHECKING:
+    import pandas as pd
+    import sqlalchemy
 from quantrocket.houston import houston
 from quantrocket.exceptions import DataInsertionError
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
 
 __all__ = [
     "list_databases",
@@ -30,13 +76,18 @@ __all__ = [
     "insert_or_ignore",
 ]
 
-def list_databases(services=None, codes=None, detail=False, expand=False):
+def list_databases(
+    services: Union[list[str], str] = None,
+    codes: Union[list[str], str] = None,
+    detail: bool = False,
+    expand: bool = False
+    ) -> dict[str, list[dict[str, str]]]:
     """
     List databases.
 
     Parameters
     ----------
-    services : str, optional
+    services : list of str, optional
         limit to these services
 
     codes: list of str, optional
@@ -56,6 +107,12 @@ def list_databases(services=None, codes=None, detail=False, expand=False):
         dict of lists of databases (one key for PostgreSQL databases and one for
         SQLite databases)
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Database Management: https://qrok.it/dl/qr/db
+
     Examples
     --------
     Load database details in a pandas DataFrame:
@@ -82,16 +139,20 @@ def list_databases(services=None, codes=None, detail=False, expand=False):
 def _cli_list_databases(*args, **kwargs):
     return json_to_cli(list_databases, *args, **kwargs)
 
-def get_s3_config():
+def get_s3_config() -> dict[str, str]:
     """
     Return the current S3 configuration, if any.
 
-    See http://qrok.it/h/dbs3 to learn more.
-
     Returns
     -------
     dict
         configuration details
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Amazon S3: http://qrok.it/dl/qr/dbs3
     """
     response = houston.get("/db/s3config")
     houston.raise_for_status_with_json(response)
@@ -100,13 +161,16 @@ def get_s3_config():
         return {}
     return response.json()
 
-def set_s3_config(access_key_id=None, secret_access_key=None, bucket=None, region=None):
+def set_s3_config(
+    access_key_id: str = None,
+    secret_access_key: str = None,
+    bucket: str = None,
+    region: str = None
+    ) -> dict[str, str]:
     """
     Set AWS S3 configuration for pushing and pulling databases to and from
     S3.
 
-    See http://qrok.it/h/dbs3 to learn more.
-
     Credentials are encrypted at rest and never leave your deployment.
 
     Parameters
@@ -129,6 +193,12 @@ def set_s3_config(access_key_id=None, secret_access_key=None, bucket=None, regio
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Amazon S3: http://qrok.it/dl/qr/dbs3
     """
     if access_key_id and not secret_access_key:
         secret_access_key = getpass.getpass(prompt="Enter AWS Secret Access Key: ")
@@ -154,12 +224,13 @@ def _cli_get_or_set_s3_config(access_key_id=None, secret_access_key=None,
     else:
         return json_to_cli(get_s3_config, *args, **kwargs)
 
-def s3_push_databases(services=None, codes=None):
+def s3_push_databases(
+    services: Union[list[str], str] = None,
+    codes: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Push database(s) to Amazon S3.
 
-    See http://qrok.it/h/dbs3 to learn more.
-
     Parameters
     ----------
     serivces : list of str, optional
@@ -170,8 +241,14 @@ def s3_push_databases(services=None, codes=None):
 
     Returns
     -------
-    json
+    dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Amazon S3: http://qrok.it/dl/qr/dbs3
     """
     params = {}
     if services:
@@ -185,12 +262,14 @@ def s3_push_databases(services=None, codes=None):
 def _cli_s3_push_databases(*args, **kwargs):
     return json_to_cli(s3_push_databases, *args, **kwargs)
 
-def s3_pull_databases(services=None, codes=None, force=False):
+def s3_pull_databases(
+    services: Union[list[str], str] = None,
+    codes: Union[list[str], str] = None,
+    force: bool = False
+    ) -> dict[str, str]:
     """
     Pull database(s) from Amazon S3.
 
-    See http://qrok.it/h/dbs3 to learn more.
-
     Parameters
     ----------
     serivces : list of str, optional
@@ -205,8 +284,14 @@ def s3_pull_databases(services=None, codes=None, force=False):
 
     Returns
     -------
-    json
+    dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Amazon S3: http://qrok.it/dl/qr/dbs3
     """
     params = {}
     if services:
@@ -222,7 +307,10 @@ def s3_pull_databases(services=None, codes=None, force=False):
 def _cli_s3_pull_databases(*args, **kwargs):
     return json_to_cli(s3_pull_databases, *args, **kwargs)
 
-def optimize_databases(services=None, codes=None):
+def optimize_databases(
+    services: Union[list[str], str] = None,
+    codes: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Optimize databases to improve performance.
 
@@ -239,8 +327,14 @@ def optimize_databases(services=None, codes=None):
 
     Returns
     -------
-    json
+    dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Database Management: https://qrok.it/dl/qr/db
     """
     params = {}
     if codes:
@@ -254,9 +348,11 @@ def optimize_databases(services=None, codes=None):
 def _cli_optimize_databases(*args, **kwargs):
     return json_to_cli(optimize_databases, *args, **kwargs)
 
-def connect_sqlite(db_path):
+def connect_sqlite(
+    db_path: str
+    ) -> 'sqlalchemy.engine.Engine':
     """
-    Returns a connection to a SQLite database.
+    Return a connection to a SQLite database.
 
     Parameters
     ----------
@@ -267,6 +363,12 @@ def connect_sqlite(db_path):
     -------
     sqlalchemy.engine.Engine
         database connection
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Custom Data: https://qrok.it/dl/qr/custom-data
     """
     try:
         from sqlalchemy import create_engine
@@ -345,7 +447,11 @@ DROP TABLE {temptable};
 
     os.remove(temp_file_name)
 
-def insert_or_fail(df, table_name, conn):
+def insert_or_fail(
+    df: 'pd.DataFrame',
+    table_name: str,
+    conn: 'sqlalchemy.engine.Engine'
+    ) -> None:
     """
     Insert a DataFrame into a SQLite database.
 
@@ -375,10 +481,20 @@ def insert_or_fail(df, table_name, conn):
     quantrocket.exceptions.DataInsertionError
         catch-all exception class for errors that occur when writing to the
         SQLite database
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Custom Data: https://qrok.it/dl/qr/custom-data
     """
     _insert_into(df, table_name, conn, "FAIL")
 
-def insert_or_replace(df, table_name, conn):
+def insert_or_replace(
+    df: 'pd.DataFrame',
+    table_name: str,
+    conn: 'sqlalchemy.engine.Engine'
+    ) -> None:
     """
     Insert a DataFrame into a SQLite database.
 
@@ -408,10 +524,20 @@ def insert_or_replace(df, table_name, conn):
     quantrocket.exceptions.DataInsertionError
         catch-all exception class for errors that occur when writing to the
         SQLite database
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Custom Data: https://qrok.it/dl/qr/custom-data
     """
     _insert_into(df, table_name, conn, "REPLACE")
 
-def insert_or_ignore(df, table_name, conn):
+def insert_or_ignore(
+    df: 'pd.DataFrame',
+    table_name: str,
+    conn: 'sqlalchemy.engine.Engine'
+    ) -> None:
     """
     Insert a DataFrame into a SQLite database.
 
@@ -441,5 +567,11 @@ def insert_or_ignore(df, table_name, conn):
     quantrocket.exceptions.DataInsertionError
         catch-all exception class for errors that occur when writing to the
         SQLite database
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Custom Data: https://qrok.it/dl/qr/custom-data
     """
     _insert_into(df, table_name, conn, "IGNORE")
