@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,15 +11,65 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for collecting and querying historical data.
 
+Functions
+---------
+create_edi_db
+    Create a new database for collecting historical data from EDI.
+
+create_ibkr_db
+    Create a new database for collecting historical data from Interactive Brokers.
+
+create_sharadar_db
+    Create a new database for collecting historical data from Sharadar.
+
+create_usstock_db
+    Create a new database for collecting historical US stock data from QuantRocket.
+
+create_custom_db
+    Create a new database into which custom data can be loaded.
+
+get_db_config
+    Return the configuration for a history database.
+
+drop_db
+    Delete a history database.
+
+list_databases
+    List history databases.
+
+collect_history
+    Collect historical market data from a vendor and save it to a history database.
+
+get_history_queue
+    Get the current queue of historical data collections.
+
+cancel_collections
+    Cancel running or pending historical data collections.
+
+wait_for_collections
+    Wait for historical data collection to finish.
+
+download_history_file
+    Query historical market data from a history database and download to file.
+
+Notes
+-----
+Usage Guide:
+
+* Historical Data: https://qrok.it/dl/qr/historical
+"""
 import os
 import sys
 import requests
+from quantrocket.utils._typing import FilepathOrBuffer, Union, Literal
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.stream import to_bytes
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.stream import to_bytes
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
 from quantrocket.exceptions import NoHistoricalData
 
 __all__ = [
@@ -40,7 +90,10 @@ __all__ = [
 
 TMP_DIR = os.environ.get("QUANTROCKET_TMP_DIR", "/tmp")
 
-def create_edi_db(code, exchanges):
+def create_edi_db(
+    code: str,
+    exchanges: Union[list[str], str]
+    ) -> dict[str, str]:
     """
     Create a new database for collecting historical data from EDI.
 
@@ -57,6 +110,11 @@ def create_edi_db(code, exchanges):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * EDI: https://qrok.it/dl/qr/edi-hist
     """
     params = {
         "vendor": "edi",
@@ -70,10 +128,42 @@ def create_edi_db(code, exchanges):
 def _cli_create_edi_db(*args, **kwargs):
     return json_to_cli(create_edi_db, *args, **kwargs)
 
-def create_ibkr_db(code, universes=None, sids=None, start_date=None, end_date=None,
-                   bar_size=None, bar_type=None, outside_rth=False,
-                   primary_exchange=False, times=None, between_times=None,
-                   shard=None):
+def create_ibkr_db(
+    code: str,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    start_date: str = None,
+    end_date: str = None,
+    bar_size: Literal[
+        "1 secs", "5 secs",	"10 secs", "15 secs", "30 secs",
+        "1 min", "2 mins", "3 mins", "5 mins", "10 mins", "15 mins", "20 mins", "30 mins",
+        "1 hour", "2 hours", "3 hours", "4 hours", "8 hours",
+        "1 day",
+        "1 week",
+        "1 month"] = None,
+    bar_type: Literal[
+        "TRADES",
+        "ADJUSTED_LAST",
+        "MIDPOINT",
+        "BID",
+        "ASK",
+        "BID_ASK",
+        "HISTORICAL_VOLATILITY",
+        "OPTION_IMPLIED_VOLATILITY"
+    ] = None,
+    outside_rth: bool = False,
+    primary_exchange: bool = False,
+    times: Union[list[str], str] = None,
+    between_times: Union[list[str], str] = None,
+    shard: Literal[
+        "year",
+        "month",
+        "day",
+        "time",
+        "sid",
+        "sid,time",
+        "off"] = None
+    ) -> dict[str, str]:
     """
     Create a new database for collecting historical data from Interactive Brokers.
 
@@ -148,6 +238,13 @@ def create_ibkr_db(code, universes=None, sids=None, start_date=None, end_date=No
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Interactive Brokers Historical Data: https://qrok.it/dl/qr/ibkr-hist
+    * Cumulative daily prices for intraday data: https://qrok.it/dl/qr/cumulative-daily
+    * Primary vs Consolidated Prices: https://qrok.it/dl/qr/primary-v-consolidated
     """
     params = {}
     if universes:
@@ -183,7 +280,11 @@ def create_ibkr_db(code, universes=None, sids=None, start_date=None, end_date=No
 def _cli_create_ibkr_db(*args, **kwargs):
     return json_to_cli(create_ibkr_db, *args, **kwargs)
 
-def create_sharadar_db(code, sec_type, country="US"):
+def create_sharadar_db(
+    code: str,
+    sec_type: Literal["STK", "ETF"],
+    country: Literal["US", "FREE"] = "US"
+    ) -> dict[str, str]:
     """
     Create a new database for collecting historical data from Sharadar.
 
@@ -203,6 +304,11 @@ def create_sharadar_db(code, sec_type, country="US"):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Sharadar Historical Data: https://qrok.it/dl/qr/sharadar-hist
     """
     params = {"vendor": "sharadar"}
     if sec_type:
@@ -218,7 +324,12 @@ def create_sharadar_db(code, sec_type, country="US"):
 def _cli_create_sharadar_db(*args, **kwargs):
     return json_to_cli(create_sharadar_db, *args, **kwargs)
 
-def create_usstock_db(code, bar_size=None, free=False, universe=None):
+def create_usstock_db(
+    code: str,
+    bar_size: Literal["1 day"] = None,
+    free: bool = False,
+    universe: str = None
+    ) -> dict[str, str]:
     """
     Create a new database for collecting historical US stock data from QuantRocket.
 
@@ -244,11 +355,17 @@ def create_usstock_db(code, bar_size=None, free=False, universe=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * US Stock Historical Data: https://qrok.it/dl/qr/usstock-hist
+
     Examples
     --------
     Create a database for end-of-day US stock prices:
 
-    create_usstock_db('usstock-1d')
+    >>> create_usstock_db('usstock-1d')
     """
     params = {
         "vendor": "usstock",
@@ -277,7 +394,16 @@ def create_usstock_db(code, bar_size=None, free=False, universe=None):
 def _cli_create_usstock_db(*args, **kwargs):
     return json_to_cli(create_usstock_db, *args, **kwargs)
 
-def create_custom_db(code, bar_size=None, columns=None):
+def create_custom_db(
+    code: str,
+    bar_size: str = None,
+    columns: dict[str, Literal[
+        "int",
+        "float",
+        "str",
+        "date",
+        "datetime"]] = None
+    ) -> dict[str, str]:
     """
     Create a new database into which custom data can be loaded.
 
@@ -304,6 +430,12 @@ def create_custom_db(code, bar_size=None, columns=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Custom Data: https://qrok.it/dl/qr/custom-data
+
     Examples
     --------
     Create a custom database for loading fundamental data:
@@ -348,7 +480,7 @@ def _cli_create_custom_db(*args, **kwargs):
         kwargs["columns"] = dict_strs_to_dict(*columns)
     return json_to_cli(create_custom_db, *args, **kwargs)
 
-def get_db_config(code):
+def get_db_config(code: str) -> dict[str, str]:
     """
     Return the configuration for a history database.
 
@@ -362,6 +494,11 @@ def get_db_config(code):
     dict
         config
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     response = houston.get("/history/databases/{0}".format(code))
     houston.raise_for_status_with_json(response)
@@ -370,7 +507,10 @@ def get_db_config(code):
 def _cli_get_db_config(*args, **kwargs):
     return json_to_cli(get_db_config, *args, **kwargs)
 
-def drop_db(code, confirm_by_typing_db_code_again=None):
+def drop_db(
+    code: str,
+    confirm_by_typing_db_code_again: str = None
+    ) -> dict[str, str]:
     """
     Delete a history database.
 
@@ -391,6 +531,11 @@ def drop_db(code, confirm_by_typing_db_code_again=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     params = {"confirm_by_typing_db_code_again": confirm_by_typing_db_code_again}
     response = houston.delete("/history/databases/{0}".format(code), params=params)
@@ -400,7 +545,7 @@ def drop_db(code, confirm_by_typing_db_code_again=None):
 def _cli_drop_db(*args, **kwargs):
     return json_to_cli(drop_db, *args, **kwargs)
 
-def list_databases():
+def list_databases() -> list[str]:
     """
     List history databases.
 
@@ -409,6 +554,11 @@ def list_databases():
     list
         list of database codes
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     response = houston.get("/history/databases")
     houston.raise_for_status_with_json(response)
@@ -417,8 +567,14 @@ def list_databases():
 def _cli_list_databases(*args, **kwargs):
     return json_to_cli(list_databases, *args, **kwargs)
 
-def collect_history(codes, sids=None, universes=None, start_date=None, end_date=None,
-                    priority=False):
+def collect_history(
+    codes: Union[list[str], str],
+    sids: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    start_date: str = None,
+    end_date: str = None,
+    priority: bool = False
+    ) -> dict[str, str]:
     """
     Collect historical market data from a vendor and save it to a history database.
 
@@ -458,6 +614,11 @@ def collect_history(codes, sids=None, universes=None, start_date=None, end_date=
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     params = {}
     if codes:
@@ -480,7 +641,7 @@ def collect_history(codes, sids=None, universes=None, start_date=None, end_date=
 def _cli_collect_history(*args, **kwargs):
     return json_to_cli(collect_history, *args, **kwargs)
 
-def get_history_queue():
+def get_history_queue() -> dict[str, str]:
     """
     Get the current queue of historical data collections.
 
@@ -489,6 +650,11 @@ def get_history_queue():
     dict
         queue by vendor
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     response = houston.get("/history/queue")
     houston.raise_for_status_with_json(response)
@@ -497,7 +663,7 @@ def get_history_queue():
 def _cli_get_history_queue(*args, **kwargs):
     return json_to_cli(get_history_queue, *args, **kwargs)
 
-def cancel_collections(codes):
+def cancel_collections(codes: Union[list[str], str]) -> dict[str, str]:
     """
     Cancel running or pending historical data collections.
 
@@ -511,6 +677,11 @@ def cancel_collections(codes):
     dict
         queue by vendor
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
     """
     params = {}
     if codes:
@@ -522,7 +693,10 @@ def cancel_collections(codes):
 def _cli_cancel_collections(*args, **kwargs):
     return json_to_cli(cancel_collections, *args, **kwargs)
 
-def wait_for_collections(codes, timeout=None):
+def wait_for_collections(
+    codes: Union[list[str], str],
+    timeout: str = None
+    ) -> dict[str, str]:
     """
     Wait for historical data collection to finish.
 
@@ -540,6 +714,11 @@ def wait_for_collections(codes, timeout=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * History database as real-time feed: https://qrok.it/dl/qr/history-wait
     """
     params = {}
     params["codes"] = codes
@@ -552,11 +731,20 @@ def wait_for_collections(codes, timeout=None):
 def _cli_wait_for_collections(*args, **kwargs):
     return json_to_cli(wait_for_collections, *args, **kwargs)
 
-def download_history_file(code, filepath_or_buffer=None, output="csv",
-                          start_date=None, end_date=None,
-                          universes=None, sids=None,
-                          exclude_universes=None, exclude_sids=None,
-                          times=None, cont_fut=None, fields=None):
+def download_history_file(
+    code: str,
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    start_date: str = None,
+    end_date: str = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    times: Union[list[str], str] = None,
+    cont_fut: Literal["concat"] = None,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Query historical market data from a history database and download to file.
 
@@ -604,6 +792,16 @@ def download_history_file(code, filepath_or_buffer=None, output="csv",
     -------
     None
 
+    See Also
+    --------
+    quantrocket.get_prices : load prices into a DataFrame
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Historical Data: https://qrok.it/dl/qr/historical
+
     Examples
     --------
     You can use StringIO to load the CSV into pandas.
@@ -611,10 +809,6 @@ def download_history_file(code, filepath_or_buffer=None, output="csv",
     >>> f = io.StringIO()
     >>> download_history_file("my-db", f)
     >>> history = pd.read_csv(f, parse_dates=["Date"])
-
-    See Also
-    --------
-    quantrocket.get_prices : load prices into a DataFrame
     """
     params = {}
     if start_date:
