@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,6 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Classes and functions related to Houston, QuantRocket's API Gateway.
+
+Classes
+-------
+Houston
+    Client interface to Houston, QuantRocket's API Gateway.
+
+Functions
+---------
+ping
+    Ping the Houston service.
+"""
 
 import os
 import six
@@ -18,7 +31,7 @@ import requests
 import re
 import uuid
 from .exceptions import ImproperlyConfigured, CannotConnectToHouston
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
 
 __all__ = [
     "Houston",
@@ -213,15 +226,19 @@ To set the environment variable on Linux, run:
 
 # Instantiate houston so that all callers can share a TCP connection (for
 # performance's sake)
-houston = Houston()
+houston: Houston = Houston()
+"""
+Houston instance for making requests to the houston API gateway. Using a single
+instance of Houston for all requests will improve performance.
+"""
 
-def ping():
+def ping() -> dict[str, str]:
     """
     Pings houston.
 
     Returns
     -------
-    json
+    dict
         reply from houston
     """
     response = houston.get("/ping")
