@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,11 +11,37 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for working with IB Gateway.
+
+Functions
+---------
+get_credentials
+    Return username and trading mode (paper/live) for IB Gateway.
+
+set_credentials
+    Set username/password and trading mode (paper/live) for IB Gateway.
+
+list_gateway_statuses
+    Query statuses of IB Gateways.
+
+start_gateways
+    Start one or more IB Gateways.
+
+stop_gateways
+    Stop one or more IB Gateways.
+
+Notes
+-----
+Usage Guide:
+
+* IB Gateway: https://qrok.it/dl/qr/ibgateway
+"""
 
-import os
 import getpass
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket.utils._typing import Union, Literal
 
 __all__ = [
     "get_credentials",
@@ -23,13 +49,11 @@ __all__ = [
     "list_gateway_statuses",
     "start_gateways",
     "stop_gateways",
-    "load_ibg_config",
-    "get_ibg_config",
 ]
 
-def get_credentials(gateway):
+def get_credentials(gateway: str) -> dict[str, str]:
     """
-    Returns username and trading mode (paper/live) for IB Gateway.
+    Return username and trading mode (paper/live) for IB Gateway.
 
     Parameters
     ----------
@@ -40,6 +64,12 @@ def get_credentials(gateway):
     -------
     dict
         credentials
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IB Gateway: https://qrok.it/dl/qr/ibgateway
     """
     statuses = list_gateway_statuses(gateways=[gateway])
     if not statuses:
@@ -52,7 +82,12 @@ def get_credentials(gateway):
         return {}
     return response.json()
 
-def set_credentials(gateway, username=None, password=None, trading_mode=None):
+def set_credentials(
+    gateway: str,
+    username: str = None,
+    password: str = None,
+    trading_mode: Literal["paper", "live"] = None
+    ) -> dict[str, str]:
     """
     Set username/password and trading mode (paper/live) for IB Gateway.
 
@@ -81,6 +116,12 @@ def set_credentials(gateway, username=None, password=None, trading_mode=None):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IB Gateway: https://qrok.it/dl/qr/ibgateway
     """
     statuses = list_gateway_statuses(gateways=[gateway])
     if not statuses:
@@ -107,7 +148,10 @@ def _cli_get_or_set_credentials(*args, **kwargs):
     else:
         return json_to_cli(get_credentials, gateway=kwargs.get("gateway", None))
 
-def list_gateway_statuses(status=None, gateways=None):
+def list_gateway_statuses(
+    status: Literal["running", "stopped", "error"] = None,
+    gateways: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Query statuses of IB Gateways.
 
@@ -122,6 +166,12 @@ def list_gateway_statuses(status=None, gateways=None):
     Returns
     -------
     dict of gateway:status (if status arg not provided), or list of gateways (if status arg provided)
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IB Gateway: https://qrok.it/dl/qr/ibgateway
     """
     params = {}
     if gateways:
@@ -136,7 +186,10 @@ def list_gateway_statuses(status=None, gateways=None):
 def _cli_list_gateway_statuses(*args, **kwargs):
     return json_to_cli(list_gateway_statuses, *args, **kwargs)
 
-def start_gateways(gateways=None, wait=False):
+def start_gateways(
+    gateways: Union[list[str], str] = None,
+    wait: bool = False
+    ) -> dict[str, str]:
     """
     Start one or more IB Gateways.
 
@@ -153,19 +206,29 @@ def start_gateways(gateways=None, wait=False):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IB Gateway: https://qrok.it/dl/qr/ibgateway
     """
     params = {"wait": wait}
     if gateways:
         params["gateways"] = gateways
 
-    response = houston.post("/ibgrouter/gateways", params=params, timeout=120)
+    # use a long timeout in case the user takes a long time to see the 2FA notifications
+    response = houston.post("/ibgrouter/gateways", params=params, timeout=60*60*24)
     houston.raise_for_status_with_json(response)
     return response.json()
 
 def _cli_start_gateways(*args, **kwargs):
     return json_to_cli(start_gateways, *args, **kwargs)
 
-def stop_gateways(gateways=None, wait=False):
+def stop_gateways(
+    gateways: Union[list[str], str] = None,
+    wait: bool = False
+    ) -> dict[str, str]:
     """
     Stop one or more IB Gateways.
 
@@ -182,6 +245,12 @@ def stop_gateways(gateways=None, wait=False):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IB Gateway: https://qrok.it/dl/qr/ibgateway
     """
     params = {"wait": wait}
     if gateways:
@@ -194,7 +263,7 @@ def stop_gateways(gateways=None, wait=False):
 def _cli_stop_gateways(*args, **kwargs):
     return json_to_cli(stop_gateways, *args, **kwargs)
 
-def load_ibg_config(filename):
+def load_ibg_config(filename: str) -> dict[str, str]:
     """
     Upload a new IB Gateway permissions config.
 
@@ -216,9 +285,9 @@ def load_ibg_config(filename):
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def get_ibg_config():
+def get_ibg_config() -> dict[str, str]:
     """
-    Returns the current IB Gateway permissions config.
+    Return the current IB Gateway permissions config.
 
     Returns
     -------
