@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,10 +11,47 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for setting and viewing software licenses and third-party API
+keys.
 
+Functions
+---------
+get_license_profile
+    Return the current license profile.
+
+set_license
+    Set QuantRocket license key.
+
+get_alpaca_key
+    Returns the current API key(s) for Alpaca.
+
+set_alpaca_key
+    Set Alpaca API key.
+
+get_polygon_key
+    Returns the current API key for Polygon.
+
+set_polygon_key
+    Set Polygon API key.
+
+get_quandl_key
+    Returns the current API key for Quandl.
+
+set_quandl_key
+    Set Quandl API key.
+
+Notes
+-----
+Usage Guide:
+
+* License Key: https://qrok.it/dl/qr/license
+* Broker and Data Connections: https://qrok.it/dl/qr/connect
+"""
 import getpass
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket.utils._typing import Literal
 
 __all__ = [
     "get_license_profile",
@@ -27,7 +64,7 @@ __all__ = [
     "set_quandl_key",
 ]
 
-def get_license_profile(force_refresh=False):
+def get_license_profile(force_refresh: bool = False) -> dict[str, str]:
     """
     Return the current license profile.
 
@@ -41,6 +78,12 @@ def get_license_profile(force_refresh=False):
     -------
     dict
         license profile
+
+    Notes
+    -----
+    Usage Guide:
+
+    * License Key: https://qrok.it/dl/qr/license
     """
     params = {}
     if force_refresh:
@@ -53,7 +96,7 @@ def get_license_profile(force_refresh=False):
 def _cli_get_license_profile(*args, **kwargs):
     return json_to_cli(get_license_profile, *args, **kwargs)
 
-def set_license(key):
+def set_license(key: str) -> dict[str, str]:
     """
     Set QuantRocket license key.
 
@@ -66,6 +109,12 @@ def set_license(key):
     -------
     dict
         license profile
+
+    Notes
+    -----
+    Usage Guide:
+
+    * License Key: https://qrok.it/dl/qr/license
     """
     response = houston.put("/license-service/license/{0}".format(key))
     houston.raise_for_status_with_json(response)
@@ -74,7 +123,7 @@ def set_license(key):
 def _cli_set_license(*args, **kwargs):
     return json_to_cli(set_license, *args, **kwargs)
 
-def get_alpaca_key():
+def get_alpaca_key() -> dict[str, str]:
     """
     Returns the current API key(s) for Alpaca.
 
@@ -82,6 +131,12 @@ def get_alpaca_key():
     -------
     dict
         credentials
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     response = houston.get("/license-service/credentials/alpaca")
     houston.raise_for_status_with_json(response)
@@ -90,7 +145,12 @@ def get_alpaca_key():
         return {}
     return response.json()
 
-def set_alpaca_key(api_key, trading_mode, secret_key=None, realtime_data="iex"):
+def set_alpaca_key(
+    api_key: str,
+    trading_mode: Literal["paper", "live"],
+    secret_key: str = None,
+    realtime_data: Literal["iex", "sip"] = "iex"
+    ) -> dict[str, str]:
     """
     Set Alpaca API key.
 
@@ -116,6 +176,12 @@ def set_alpaca_key(api_key, trading_mode, secret_key=None, realtime_data="iex"):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     if not secret_key:
         secret_key = getpass.getpass(prompt="Enter Alpaca secret key: ")
@@ -136,7 +202,7 @@ def _cli_get_or_set_alpaca_key(*args, **kwargs):
     else:
         return json_to_cli(get_alpaca_key)
 
-def get_polygon_key():
+def get_polygon_key() -> dict[str, str]:
     """
     Returns the current API key for Polygon.
 
@@ -144,6 +210,12 @@ def get_polygon_key():
     -------
     dict
         credentials
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     response = houston.get("/license-service/credentials/polygon")
     houston.raise_for_status_with_json(response)
@@ -152,7 +224,7 @@ def get_polygon_key():
         return {}
     return response.json()
 
-def set_polygon_key(api_key):
+def set_polygon_key(api_key: str) -> dict[str, str]:
     """
     Set Polygon API key.
 
@@ -168,6 +240,12 @@ def set_polygon_key(api_key):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     data = {}
     data["api_key"] = api_key
@@ -182,7 +260,7 @@ def _cli_get_or_set_polygon_key(*args, **kwargs):
     else:
         return json_to_cli(get_polygon_key)
 
-def get_quandl_key():
+def get_quandl_key() -> dict[str, str]:
     """
     Returns the current API key for Quandl.
 
@@ -190,6 +268,12 @@ def get_quandl_key():
     -------
     dict
         credentials
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     response = houston.get("/license-service/credentials/quandl")
     houston.raise_for_status_with_json(response)
@@ -198,7 +282,7 @@ def get_quandl_key():
         return {}
     return response.json()
 
-def set_quandl_key(api_key):
+def set_quandl_key(api_key: str) -> dict[str, str]:
     """
     Set Quandl API key.
 
@@ -214,6 +298,12 @@ def set_quandl_key(api_key):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Broker and Data Connections: https://qrok.it/dl/qr/connect
     """
     data = {}
     data["api_key"] = api_key
