@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,15 +11,104 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for working with the securities master database.
 
+Functions
+---------
+list_ibkr_exchanges
+    List exchanges by security type and country as found on the IBKR website.
+
+collect_alpaca_listings
+    Collect securities listings from Alpaca and store in securities master
+    database.
+
+collect_edi_listings
+    Collect securities listings from EDI and store in securities master
+    database.
+
+collect_figi_listings
+    Collect securities listings from Bloomberg OpenFIGI and store
+    in securities master database.
+
+collect_ibkr_listings
+    Collect securities listings from Interactive Brokers and store in
+    securities master database.
+
+collect_sharadar_listings
+    Collect securities listings from Sharadar and store in securities master
+    database.
+
+collect_usstock_listings
+    Collect US stock listings from QuantRocket and store in securities
+    master database.
+
+collect_ibkr_option_chains
+    Collect IBKR option chains for underlying securities.
+
+diff_ibkr_securities
+    Flag security details that have changed in IBKR's system since the time they
+    were last collected into the securities master database.
+
+download_master_file
+    Query security details from the securities master database and download to file.
+
+get_securities
+    Return a DataFrame of security details from the securities master database.
+
+get_securities_reindexed_like
+    Return a multiindex DataFrame of securities master data, reindexed to
+    match the index and columns (sids) of `reindex_like`.
+
+get_contract_nums_reindexed_like
+    From a DataFrame of futures (with dates as the index and sids as columns),
+    return a DataFrame of integers representing each sid's sequence in the
+    futures chain as of each date, where 1 is the front contract, 2 is the second
+    nearest contract, etc.
+
+create_universe
+    Create a universe of securities.
+
+delete_universe
+    Delete a universe.
+
+list_universes
+    List universes and their size.
+
+delist_ibkr_security
+    Mark an IBKR security as delisted.
+
+create_ibkr_combo
+    Create an IBKR combo (aka spread).
+
+collect_ibkr_calendar
+    Collect upcoming trading hours from IBKR for exchanges and save to
+    securities master database.
+
+list_calendar_statuses
+    Check whether exchanges are open or closed.
+
+round_to_tick_sizes
+    Round prices in a CSV file to valid tick sizes.
+
+Notes
+-----
+Usage Guide:
+
+* Securities Master: https://qrok.it/dl/qr/master
+"""
 import sys
 import six
 import json
 import requests
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import pandas as pd
+from quantrocket.utils._typing import FilepathOrBuffer, Union, Literal
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.stream import to_bytes
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.stream import to_bytes
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
 from quantrocket.exceptions import ParameterError, NoMasterData
 
 __all__ = [
@@ -41,14 +130,19 @@ __all__ = [
     "list_universes",
     "delist_ibkr_security",
     "create_ibkr_combo",
-    "load_rollrules_config",
-    "get_rollrules_config",
     "collect_ibkr_calendar",
     "list_calendar_statuses",
     "round_to_tick_sizes",
 ]
 
-def list_ibkr_exchanges(regions=None, sec_types=None):
+def list_ibkr_exchanges(
+    regions: Union[list[
+        Literal["north_america", "europe", "asia", "global"]],
+        Literal["north_america", "europe", "asia", "global"]] = None,
+    sec_types: Union[list[
+        Literal["STK", "ETF", "FUT", "CASH", "IND"]],
+        Literal["STK", "ETF", "FUT", "CASH", "IND"]] = None
+    ) -> dict[str, str]:
     """
     List exchanges by security type and country as found on the IBKR website.
 
@@ -63,6 +157,12 @@ def list_ibkr_exchanges(regions=None, sec_types=None):
     Returns
     -------
     dict
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Interactive Brokers listings: https://qrok.it/dl/qr/master-ibkr
     """
     params = {}
     if sec_types:
@@ -77,7 +177,7 @@ def list_ibkr_exchanges(regions=None, sec_types=None):
 def _cli_list_ibkr_exchanges(*args, **kwargs):
     return json_to_cli(list_ibkr_exchanges, *args, **kwargs)
 
-def collect_alpaca_listings():
+def collect_alpaca_listings() -> dict[str, str]:
     """
     Collect securities listings from Alpaca and store in securities master
     database.
@@ -86,6 +186,12 @@ def collect_alpaca_listings():
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Alpaca listings: https://qrok.it/dl/qr/master-alpaca
     """
     response = houston.post("/master/securities/alpaca")
     houston.raise_for_status_with_json(response)
@@ -94,7 +200,7 @@ def collect_alpaca_listings():
 def _cli_collect_alpaca_listings(*args, **kwargs):
     return json_to_cli(collect_alpaca_listings, *args, **kwargs)
 
-def collect_edi_listings(exchanges=None):
+def collect_edi_listings(exchanges: Union[list[str], str] = None) -> dict[str, str]:
     """
     Collect securities listings from EDI and store in securities master
     database.
@@ -109,6 +215,12 @@ def collect_edi_listings(exchanges=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * EDI listings: https://qrok.it/dl/qr/master-edi
+
     Examples
     --------
     Collect sample listings:
@@ -134,7 +246,7 @@ def collect_edi_listings(exchanges=None):
 def _cli_collect_edi_listings(*args, **kwargs):
     return json_to_cli(collect_edi_listings, *args, **kwargs)
 
-def collect_figi_listings():
+def collect_figi_listings() -> dict[str, str]:
     """
     Collect securities listings from Bloomberg OpenFIGI and store
     in securities master database.
@@ -155,6 +267,12 @@ def collect_figi_listings():
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * FIGI listings: https://qrok.it/dl/qr/master-figi
+
     Examples
     --------
     Collect all available FIGI listings:
@@ -168,8 +286,16 @@ def collect_figi_listings():
 def _cli_collect_figi_listings(*args, **kwargs):
     return json_to_cli(collect_figi_listings, *args, **kwargs)
 
-def collect_ibkr_listings(exchanges=None, sec_types=None, currencies=None,
-                          symbols=None, universes=None, sids=None):
+def collect_ibkr_listings(
+    exchanges: Union[list[str], str] = None,
+    sec_types: Union[list[
+        Literal["STK", "ETF", "FUT", "CASH", "IND"]],
+        Literal["STK", "ETF", "FUT", "CASH", "IND"]] = None,
+    currencies: Union[list[str], str] = None,
+    symbols: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Collect securities listings from Interactive Brokers and store in
     securities master database.
@@ -206,6 +332,12 @@ def collect_ibkr_listings(exchanges=None, sec_types=None, currencies=None,
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Interactive Brokers listings: https://qrok.it/dl/qr/master-ibkr
+
     Examples
     --------
     Collect free sample listings:
@@ -249,20 +381,26 @@ def collect_ibkr_listings(exchanges=None, sec_types=None, currencies=None,
 def _cli_collect_ibkr_listings(*args, **kwargs):
     return json_to_cli(collect_ibkr_listings, *args, **kwargs)
 
-def collect_sharadar_listings(countries="US"):
+def collect_sharadar_listings(countries: Literal["US", "FREE"] = "US") -> dict[str, str]:
     """
     Collect securities listings from Sharadar and store in securities master
     database.
 
     Parameters
     ----------
-    countries : list of str, required
-        countries to collect listings for. Possible choices: US, FREE
+    countries : str, required
+        country to collect listings for. Possible choices: US, FREE
 
     Returns
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Sharadar listings: https://qrok.it/dl/qr/master-sharadar
     """
     params = {}
     if countries:
@@ -275,7 +413,7 @@ def collect_sharadar_listings(countries="US"):
 def _cli_collect_sharadar_listings(*args, **kwargs):
     return json_to_cli(collect_sharadar_listings, *args, **kwargs)
 
-def collect_usstock_listings():
+def collect_usstock_listings() -> dict[str, str]:
     """
     Collect US stock listings from QuantRocket and store in securities
     master database.
@@ -284,6 +422,12 @@ def collect_usstock_listings():
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * US Stock listings: https://qrok.it/dl/qr/master-usstock
     """
     response = houston.post("/master/securities/usstock")
     houston.raise_for_status_with_json(response)
@@ -292,7 +436,11 @@ def collect_usstock_listings():
 def _cli_collect_usstock_listings(*args, **kwargs):
     return json_to_cli(collect_usstock_listings, *args, **kwargs)
 
-def collect_ibkr_option_chains(universes=None, sids=None, infilepath_or_buffer=None):
+def collect_ibkr_option_chains(
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    infilepath_or_buffer: FilepathOrBuffer = None
+    ) -> dict[str, str]:
     """
     Collect IBKR option chains for underlying securities.
 
@@ -317,6 +465,12 @@ def collect_ibkr_option_chains(universes=None, sids=None, infilepath_or_buffer=N
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * IBKR Option Chains: https://qrok.it/dl/qr/option-chains
     """
     params = {}
     if universes:
@@ -344,9 +498,15 @@ def collect_ibkr_option_chains(universes=None, sids=None, infilepath_or_buffer=N
 def _cli_collect_ibkr_option_chains(*args, **kwargs):
     return json_to_cli(collect_ibkr_option_chains, *args, **kwargs)
 
-def diff_ibkr_securities(universes=None, sids=None, infilepath_or_buffer=None,
-                         fields=None, delist_missing=False, delist_exchanges=None,
-                         wait=False):
+def diff_ibkr_securities(
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    infilepath_or_buffer: FilepathOrBuffer = None,
+    fields: Union[list[str], str] = None,
+    delist_missing: bool = False,
+    delist_exchanges: Union[list[str], str] = None,
+    wait: bool = False
+    ) -> dict[str, str]:
     """
     Flag security details that have changed in IBKR's system since the time they
     were last collected into the securities master database.
@@ -383,6 +543,11 @@ def diff_ibkr_securities(universes=None, sids=None, infilepath_or_buffer=None,
     dict
         dict of sids and fields that have changed (if wait), or status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Maintain listings: https://qrok.it/dl/qr/maintain-listings
     """
     params = {}
     if universes:
@@ -420,11 +585,27 @@ def diff_ibkr_securities(universes=None, sids=None, infilepath_or_buffer=None,
 def _cli_diff_ibkr_securities(*args, **kwargs):
     return json_to_cli(diff_ibkr_securities, *args, **kwargs)
 
-def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None, sec_types=None,
-                         currencies=None, universes=None, symbols=None, sids=None,
-                         exclude_universes=None, exclude_sids=None,
-                         exclude_delisted=False, exclude_expired=False,
-                         frontmonth=False, vendors=None, fields=None):
+def download_master_file(
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    exchanges: Union[list[str], str] = None,
+    sec_types: Union[list[
+        Literal["STK", "ETF", "FUT", "CASH", "IND", "OPT", "FOP", "BAG"]],
+        Literal["STK", "ETF", "FUT", "CASH", "IND", "OPT", "FOP", "BAG"]] = None,
+    currencies: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    symbols: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    exclude_delisted: bool = False,
+    exclude_expired: bool = False,
+    frontmonth: bool = False,
+    vendors: Union[list[
+        Literal["alpaca", "edi", "ibkr", "sharadar", "usstock"]],
+        Literal["alpaca", "edi", "ibkr", "sharadar", "usstock"]] = None,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Query security details from the securities master database and download to file.
 
@@ -488,6 +669,10 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
     -------
     None
 
+    See Also
+    --------
+    get_securities : load securities into a DataFrame
+
     Notes
     -----
     Parameters for filtering query results are combined according to the following
@@ -507,6 +692,10 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
     set. Exclusion filters are ORed, that is, securities are excluded if they match any
     of the exclusion criteria.
 
+    Usage Guide:
+
+    * Master file: https://qrok.it/dl/qr/master-file
+
     Examples
     --------
     Download NYSE and NASDAQ securities to file, using MICs to specify
@@ -525,10 +714,6 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
     >>> f = io.StringIO()
     >>> download_master_file(f, fields="*", universes="my-universe")
     >>> securities = pd.read_csv(f)
-
-    See Also
-    --------
-    get_securities : load securities into a DataFrame
     """
     params = {}
     if exchanges:
@@ -582,11 +767,25 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
 def _cli_download_master_file(*args, **kwargs):
     return json_to_cli(download_master_file, *args, **kwargs)
 
-def get_securities(symbols=None, exchanges=None, sec_types=None,
-                   currencies=None, universes=None, sids=None,
-                   exclude_universes=None, exclude_sids=None,
-                   exclude_delisted=False, exclude_expired=False,
-                   frontmonth=False, vendors=None, fields=None):
+def get_securities(
+    symbols: Union[list[str], str] = None,
+    exchanges: Union[list[str], str] = None,
+    sec_types: Union[list[
+        Literal["STK", "ETF", "FUT", "CASH", "IND", "OPT", "FOP", "BAG"]],
+        Literal["STK", "ETF", "FUT", "CASH", "IND", "OPT", "FOP", "BAG"]] = None,
+    currencies: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    exclude_delisted: bool = False,
+    exclude_expired: bool = False,
+    frontmonth: bool = False,
+    vendors: Union[list[
+        Literal["alpaca", "edi", "ibkr", "sharadar", "usstock"]],
+        Literal["alpaca", "edi", "ibkr", "sharadar", "usstock"]] = None,
+    fields: Union[list[str], str] = None
+    ) -> 'pd.DataFrame':
     """
     Return a DataFrame of security details from the securities master database.
 
@@ -664,6 +863,10 @@ def get_securities(symbols=None, exchanges=None, sec_types=None,
     set. Exclusion filters are ORed, that is, securities are excluded if they match any
     of the exclusion criteria.
 
+    Usage Guide:
+
+    * Master file: https://qrok.it/dl/qr/master-file
+
     Examples
     --------
     Load default fields for NYSE and NASDAQ securities, using MICs to specify
@@ -715,7 +918,10 @@ def get_securities(symbols=None, exchanges=None, sec_types=None,
 
     return securities
 
-def get_securities_reindexed_like(reindex_like, fields=None):
+def get_securities_reindexed_like(
+    reindex_like: 'pd.DataFrame',
+    fields: Union[list[str], str] = None
+    ) -> 'pd.DataFrame':
     """
     Return a multiindex DataFrame of securities master data, reindexed to
     match the index and columns (sids) of `reindex_like`.
@@ -743,6 +949,13 @@ def get_securities_reindexed_like(reindex_like, fields=None):
         a multiindex (Field, Date) DataFrame of securities master data, shaped
         like the input DataFrame
 
+    Notes
+    -----
+    Usage Guide:
+
+    * get_securities_reindexed_like: https://qrok.it/dl/qr/master-reindex
+    * Master file: https://qrok.it/dl/qr/master-file
+
     Examples
     --------
     Get exchanges (MICs) using a DataFrame of prices:
@@ -778,7 +991,10 @@ def get_securities_reindexed_like(reindex_like, fields=None):
     securities = securities.reindex(columns=reindex_like.columns)
     return securities
 
-def get_contract_nums_reindexed_like(reindex_like, limit=5):
+def get_contract_nums_reindexed_like(
+    reindex_like: 'pd.DataFrame',
+    limit: int = 5
+    ) -> 'pd.DataFrame':
     """
     From a DataFrame of futures (with dates as the index and sids as columns),
     return a DataFrame of integers representing each sid's sequence in the
@@ -805,6 +1021,12 @@ def get_contract_nums_reindexed_like(reindex_like, limit=5):
         a DataFrame of futures chain sequence numbers, shaped like the input
         DataFrame
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Futures contract numbers: https://qrok.it/dl/qr/futures-contract-nums
+
     Examples
     --------
     Get a Boolean mask of front-month contracts:
@@ -890,8 +1112,15 @@ def get_contract_nums_reindexed_like(reindex_like, limit=5):
 
     return contract_nums
 
-def create_universe(code, infilepath_or_buffer=None, sids=None, from_universes=None,
-                    exclude_delisted=False, append=False, replace=False):
+def create_universe(
+    code: str,
+    infilepath_or_buffer: FilepathOrBuffer = None,
+    sids: Union[list[str], str] = None,
+    from_universes: Union[list[str], str] = None,
+    exclude_delisted: bool = False,
+    append: bool = False,
+    replace: bool = False
+    ) -> dict[str, str]:
     """
     Create a universe of securities.
 
@@ -924,6 +1153,12 @@ def create_universe(code, infilepath_or_buffer=None, sids=None, from_universes=N
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Universes: https://qrok.it/dl/qr/universes
+
     Examples
     --------
     Create a universe called 'nyse-stk' from a CSV file:
@@ -975,7 +1210,7 @@ def create_universe(code, infilepath_or_buffer=None, sids=None, from_universes=N
 def _cli_create_universe(*args, **kwargs):
     return json_to_cli(create_universe, *args, **kwargs)
 
-def delete_universe(code):
+def delete_universe(code: str) -> dict[str, str]:
     """
     Delete a universe.
 
@@ -991,6 +1226,12 @@ def delete_universe(code):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Universes: https://qrok.it/dl/qr/universes
     """
 
     url = "/master/universes/{0}".format(code)
@@ -1002,7 +1243,7 @@ def delete_universe(code):
 def _cli_delete_universe(*args, **kwargs):
     return json_to_cli(delete_universe, *args, **kwargs)
 
-def list_universes():
+def list_universes() -> dict[str, int]:
     """
     List universes and their size.
 
@@ -1010,6 +1251,12 @@ def list_universes():
     -------
     dict
         dict of universe:size
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Universes: https://qrok.it/dl/qr/universes
     """
     response = houston.get("/master/universes")
     houston.raise_for_status_with_json(response)
@@ -1018,7 +1265,13 @@ def list_universes():
 def _cli_list_universes(*args, **kwargs):
     return json_to_cli(list_universes, *args, **kwargs)
 
-def delist_ibkr_security(sid=None, symbol=None, exchange=None, currency=None, sec_type=None):
+def delist_ibkr_security(
+    sid: str = None,
+    symbol: str = None,
+    exchange: str = None,
+    currency: str = None,
+    sec_type: Literal["STK", "ETF", "FUT", "CASH", "IND"] = None
+    ) -> dict[str, str]:
     """
     Mark an IBKR security as delisted.
 
@@ -1052,6 +1305,12 @@ def delist_ibkr_security(sid=None, symbol=None, exchange=None, currency=None, se
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Maintain listings: https://qrok.it/dl/qr/maintain-listings
     """
     params = {}
     if sid:
@@ -1072,7 +1331,9 @@ def delist_ibkr_security(sid=None, symbol=None, exchange=None, currency=None, se
 def _cli_delist_ibkr_security(*args, **kwargs):
     return json_to_cli(delist_ibkr_security, *args, **kwargs)
 
-def create_ibkr_combo(combo_legs):
+def create_ibkr_combo(
+    combo_legs: list[list[Union[str, int]]]
+    ) -> dict[str, Union[str, bool]]:
     """
     Create an IBKR combo (aka spread), which is a composite instrument consisting
     of two or more individual instruments (legs) that are traded as a single
@@ -1098,6 +1359,12 @@ def create_ibkr_combo(combo_legs):
         returns a dict containing the generated sid of the combo, and whether a new
         record was created
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Combos: https://qrok.it/dl/qr/combos
+
     Examples
     --------
     To create a calendar spread on VX, first retrieve the sids of the legs:
@@ -1129,7 +1396,7 @@ def _cli_create_ibkr_combo(combo_filepath):
         combo_legs = json.load(f)
     return json_to_cli(create_ibkr_combo, combo_legs)
 
-def load_rollrules_config(filename):
+def load_rollrules_config(filename: str) -> dict[str, str]:
     """
     Upload a new rollover rules config.
 
@@ -1148,9 +1415,9 @@ def load_rollrules_config(filename):
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def get_rollrules_config():
+def get_rollrules_config() -> dict[str, str]:
     """
-    Returns the current rollover rules config.
+    Return the current rollover rules config.
 
     Returns
     -------
@@ -1170,7 +1437,7 @@ def _cli_load_or_show_rollrules(filename=None):
     else:
         return json_to_cli(get_rollrules_config)
 
-def collect_ibkr_calendar(exchanges=None):
+def collect_ibkr_calendar(exchanges: Union[list[str], str] = None) -> dict[str, str]:
     """
     Collect upcoming trading hours from IBKR for exchanges and save to
     securities master database.
@@ -1184,6 +1451,12 @@ def collect_ibkr_calendar(exchanges=None):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Trading Calendars: https://qrok.it/dl/qr/calendars
     """
     params = {}
     if exchanges:
@@ -1196,7 +1469,13 @@ def collect_ibkr_calendar(exchanges=None):
 def _cli_collect_ibkr_calendar(*args, **kwargs):
     return json_to_cli(collect_ibkr_calendar, *args, **kwargs)
 
-def list_calendar_statuses(exchanges, sec_type=None, in_=None, ago=None, outside_rth=False):
+def list_calendar_statuses(
+    exchanges: Union[list[str], str],
+    sec_type: Literal["STK", "FUT", "CASH", "OPT"] = None,
+    in_: str = None,
+    ago: str = None,
+    outside_rth: bool = False
+    ) -> dict[str, str]:
     """
     Check whether exchanges are open or closed.
 
@@ -1225,6 +1504,12 @@ def list_calendar_statuses(exchanges, sec_type=None, in_=None, ago=None, outside
     -------
     dict
         exchange calendar status
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Trading Calendars: https://qrok.it/dl/qr/calendars
     """
     params = {}
     if exchanges:
@@ -1341,9 +1626,13 @@ def _cli_isclosed(exchanges, sec_type=None, in_=None, ago=None, since=None, unti
 
     return '', int(not is_closed)
 
-def round_to_tick_sizes(infilepath_or_buffer, round_fields,
-                        how=None, append_ticksize=False,
-                        outfilepath_or_buffer=None):
+def round_to_tick_sizes(
+    infilepath_or_buffer: FilepathOrBuffer,
+    round_fields: Union[list[str], str],
+    how: Literal['up', 'down', 'nearest'] = None,
+    append_ticksize: bool = False,
+    outfilepath_or_buffer: FilepathOrBuffer = None
+    ) -> None:
     """
     Round prices in a CSV file to valid tick sizes.
 
@@ -1371,6 +1660,12 @@ def round_to_tick_sizes(infilepath_or_buffer, round_fields,
     Returns
     -------
     None
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Tick sizes: https://qrok.it/dl/qr/tick-sizes
     """
 
     params = {}
