@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,16 +11,45 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for backtesting and trading with Moonshot.
 
+Functions
+---------
+backtest
+    Backtest one or more strategies.
+
+read_moonshot_csv
+    Load a Moonshot backtest CSV file into a DataFrame.
+
+scan_parameters
+    Run a parameter scan for one or more strategies.
+
+ml_walkforward
+    Run a walk-forward optimization of a machine learning strategy.
+
+trade
+    Run one or more strategies and generate orders.
+
+Notes
+-----
+Usage Guide:
+
+* Moonshot: https://qrok.it/dl/qr/moonshot
+"""
 import sys
 import os.path
 import six
 from zipfile import ZipFile
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import pandas as pd
+from quantrocket.utils._typing import FilepathOrBuffer, Any, Union, Literal
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
-from quantrocket.utils.parse import _read_moonshot_or_pnl_csv
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket.utils._parse import _read_moonshot_or_pnl_csv
 
 __all__ = [
     "backtest",
@@ -30,9 +59,19 @@ __all__ = [
     "trade",
 ]
 
-def backtest(strategies, start_date=None, end_date=None, segment=None, allocations=None,
-             nlv=None, params=None, details=None, output="csv", filepath_or_buffer=None,
-             no_cache=False):
+def backtest(
+    strategies: Union[list[str], str],
+    start_date: str = None,
+    end_date: str = None,
+    segment: str = None,
+    allocations: dict[str, float] = None,
+    nlv: dict[str, float] = None,
+    params: dict[str, Any] = None,
+    details: bool = None,
+    output: Literal["csv", "pdf"] = "csv",
+    filepath_or_buffer: FilepathOrBuffer = None,
+    no_cache: bool = False
+    ) -> None:
     """
     Backtest one or more strategies.
 
@@ -79,7 +118,7 @@ def backtest(strategies, start_date=None, end_date=None, segment=None, allocatio
     output : str, required
         the output format (choices are csv or pdf)
 
-    filepath_or_buffer : str, optional
+    filepath_or_buffer : str or file-like object, optional
         the location to write the results file (omit to write to stdout)
 
     no_cache : bool
@@ -91,6 +130,16 @@ def backtest(strategies, start_date=None, end_date=None, segment=None, allocatio
     -------
     None
 
+    See Also
+    --------
+    read_moonshot_csv : load a Moonshot backtest CSV into a DataFrame
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Moonshot Backtesting: https://qrok.it/dl/qr/moonshot-backtest
+
     Examples
     --------
     Backtest several HML (High Minus Low) strategies from 2005-2015 and return a
@@ -108,10 +157,6 @@ def backtest(strategies, start_date=None, end_date=None, segment=None, allocatio
                  end_date="2018-01-01",
                  segment="A",
                  filepath_or_buffer="results.csv")
-
-    See Also
-    --------
-    read_moonshot_csv : load a Moonshot backtest CSV into a DataFrame
     """
     output = output or "csv"
 
@@ -159,7 +204,9 @@ def _cli_backtest(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(backtest, *args, **kwargs)
 
-def read_moonshot_csv(filepath_or_buffer):
+def read_moonshot_csv(
+    filepath_or_buffer: FilepathOrBuffer
+    ) -> 'pd.DataFrame':
     """
     Load a Moonshot backtest CSV into a DataFrame.
 
@@ -177,6 +224,12 @@ def read_moonshot_csv(filepath_or_buffer):
         a multi-index (Field, Date[, Time]) DataFrame of backtest
         results, with sids or strategy codes as columns
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Moonshot Backtesting: https://qrok.it/dl/qr/moonshot-backtest
+
     Examples
     --------
     >>> results = read_moonshot_csv("moonshot_backtest.csv")
@@ -184,10 +237,23 @@ def read_moonshot_csv(filepath_or_buffer):
     """
     return _read_moonshot_or_pnl_csv(filepath_or_buffer)
 
-def scan_parameters(strategies, start_date=None, end_date=None, segment=None,
-                    param1=None, vals1=None, param2=None, vals2=None,
-                    allocations=None, nlv=None, params=None, num_workers=None,
-                    output="csv", filepath_or_buffer=None, no_cache=False):
+def scan_parameters(
+    strategies: Union[list[str], str],
+    start_date: str = None,
+    end_date: str = None,
+    segment: str = None,
+    param1: str = None,
+    vals1: list[Any] = None,
+    param2: str = None,
+    vals2: list[Any] = None,
+    allocations: dict[str, float] = None,
+    nlv: dict[str, float] = None,
+    params: dict[str, Any] = None,
+    num_workers: int = None,
+    output: Literal["csv", "pdf"] = "csv",
+    filepath_or_buffer: FilepathOrBuffer = None,
+    no_cache: bool = False
+    ) -> None:
     """
     Run a parameter scan for one or more strategies.
 
@@ -259,6 +325,12 @@ def scan_parameters(strategies, start_date=None, end_date=None, segment=None,
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Moonshot Parameter Scans: https://qrok.it/dl/qr/moonshot-paramscan
+
     Examples
     --------
     Run a parameter scan for several different moving averages on a strategy
@@ -343,11 +415,24 @@ def _cli_scan_parameters(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(scan_parameters, *args, **kwargs)
 
-def ml_walkforward(strategy, start_date, end_date, train, min_train=None, rolling_train=None,
-                   model_filepath=None, force_nonincremental=None, segment=None,
-                   allocation=None, nlv=None, params=None,
-                   details=None, progress=False, filepath_or_buffer=None,
-                   no_cache=False):
+def ml_walkforward(
+    strategy: str,
+    start_date: str,
+    end_date: str,
+    train: str,
+    min_train: str = None,
+    rolling_train: str = None,
+    model_filepath: str = None,
+    force_nonincremental: bool = None,
+    segment: str = None,
+    allocation: float = None,
+    nlv: dict[str, float] = None,
+    params: dict[str, Any] = None,
+    details: bool = None,
+    progress: bool = False,
+    filepath_or_buffer: FilepathOrBuffer = None,
+    no_cache: bool = False
+    ) -> None:
     """
     Run a walk-forward optimization of a machine learning strategy.
 
@@ -441,6 +526,12 @@ def ml_walkforward(strategy, start_date, end_date, train, min_train=None, rollin
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * MoonshotML: https://qrok.it/dl/qr/moonshotml
+
     Examples
     --------
     Run a walk-forward optimization using the default model and retrain the model
@@ -580,7 +671,13 @@ def _cli_ml_walkforward(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(ml_walkforward, *args, **kwargs)
 
-def trade(strategies, accounts=None, review_date=None, output="csv", filepath_or_buffer=None):
+def trade(
+    strategies: Union[list[str], str],
+    accounts: Union[list[str], str] = None,
+    review_date: str = None,
+    output: Literal["csv", "json"] = "csv",
+    filepath_or_buffer: FilepathOrBuffer = None
+    ) -> None:
     """
     Run one or more strategies and generate orders.
 
@@ -606,6 +703,12 @@ def trade(strategies, accounts=None, review_date=None, output="csv", filepath_or
     Returns
     -------
     None
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Moonshot live trading: https://qrok.it/dl/qr/moonshot-trade
     """
     params = {}
     if strategies:
