@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,13 +11,37 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for querying price data from QuantRocket databases.
 
+Functions
+---------
+get_prices
+    Query one or more history databases, real-time aggregate databases,
+    and/or Zipline bundles and load prices into a DataFrame.
+
+get_prices_reindexed_like
+    Return a multiindex (Field, Date) DataFrame of prices for one or more history
+    databases, real-time aggregate databases, or Zipline bundles, reindexed to match
+    the index (dates) and columns (sids) of the input DataFrame.
+
+Notes
+-----
+Usage Guide:
+
+* get_prices: https://qrok.it/dl/qr/prices
+* get_prices_reindexed_like: https://qrok.it/dl/qr/prices-reindex
+"""
 import six
 import os
 import time
 import itertools
 import tempfile
 import requests
+from typing import Callable, TYPE_CHECKING
+if TYPE_CHECKING:
+    import pandas as pd
+from quantrocket.utils._typing import Union, Literal
 from quantrocket.master import download_master_file
 from quantrocket.exceptions import ParameterError, NoHistoricalData, NoRealtimeData
 from quantrocket.history import (
@@ -40,12 +64,21 @@ __all__ = [
 
 TMP_DIR = os.environ.get("QUANTROCKET_TMP_DIR", tempfile.gettempdir())
 
-def get_prices(codes, start_date=None, end_date=None,
-               universes=None, sids=None,
-               exclude_universes=None, exclude_sids=None,
-               times=None, fields=None,
-               timezone=None, infer_timezone=None,
-               cont_fut=None, data_frequency=None):
+def get_prices(
+    codes: Union[str, list[str]],
+    start_date: str = None,
+    end_date: str = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    times: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None,
+    timezone: str = None,
+    infer_timezone: bool = None,
+    cont_fut: Literal["concat"] = None,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None
+    ) -> 'pd.DataFrame':
     """
     Query one or more history databases, real-time aggregate databases,
     and/or Zipline bundles and load prices into a DataFrame.
@@ -129,6 +162,10 @@ def get_prices(codes, start_date=None, end_date=None,
     applying the `times` filter as described, users can specify the times in the timezone of
     the relevant exchange for both types of databases.
 
+    Usage Guide:
+
+    * get_prices: https://qrok.it/dl/qr/prices
+
     Examples
     --------
     Load intraday prices:
@@ -531,12 +568,20 @@ def get_prices(codes, start_date=None, end_date=None,
 
     return prices
 
-def get_prices_reindexed_like(reindex_like, codes, fields=None,
-                              shift=1, ffill=True, lookback_window=10,
-                              agg="last",
-                              timezone=None, infer_timezone=None,
-                              times=None, cont_fut=None,
-                              data_frequency=None):
+def get_prices_reindexed_like(
+    reindex_like: 'pd.DataFrame',
+    codes: Union[str, list[str]],
+    fields: Union[list[str], str] = None,
+    shift: int = 1,
+    ffill: bool = True,
+    lookback_window: int = 10,
+    agg: Union[str, Callable[['pd.DataFrame'], 'pd.DataFrame']] = "last",
+    timezone: str = None,
+    infer_timezone: bool = None,
+    times: Union[list[str], str] = None,
+    cont_fut: Literal["concat"] = None,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None
+    ) -> 'pd.DataFrame':
     """
     Return a multiindex (Field, Date) DataFrame of prices for one or more history
     databases, real-time aggregate databases, or Zipline bundles, reindexed to match
@@ -640,6 +685,12 @@ def get_prices_reindexed_like(reindex_like, codes, fields=None,
     DataFrame
         a MultiIndex DataFrame of prices shaped like reindex_like
 
+    Notes
+    -----
+    Usage Guide:
+
+    * get_prices_reindexed_like: https://qrok.it/dl/qr/prices-reindex
+
     Examples
     --------
     Use a DataFrame of closing prices to query a DataFrame of fundamentals in a
