@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,16 +11,66 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for collecting and querying real-time data.
 
+Functions
+---------
+create_ibkr_tick_db
+    Create a new database for collecting real-time tick data from Interactive
+    Brokers.
+
+create_polygon_tick_db
+    Create a new database for collecting real-time tick data from Polygon.
+
+create_alpaca_tick_db
+    Create a new database for collecting real-time tick data from Alpaca.
+
+create_agg_db
+    Create an aggregate database from a tick database.
+
+get_db_config
+    Return the configuration for a tick database or aggregate database.
+
+drop_db
+    Delete a tick database or aggregate database.
+
+drop_ticks
+    Delete ticks from a tick database. Does not delete any aggregate
+    database records.
+
+list_databases
+    List tick databases and associated aggregate databases.
+
+collect_market_data
+    Collect real-time market data and save it to a tick database.
+
+get_active_collections
+    Return the number of tickers currently being collected, by vendor and
+    database.
+
+cancel_market_data
+    Cancel market data collection.
+
+download_market_data_file
+    Query market data from a tick database or aggregate database and download to file.
+
+Notes
+-----
+Usage Guide:
+
+* Real-time Data: https://qrok.it/dl/qr/realtime
+"""
 import sys
 import requests
 import urllib.parse
 import subprocess
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket.utils._typing import FilepathOrBuffer, Union, Literal
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
 from quantrocket.houston import houston
 from quantrocket.exceptions import NoRealtimeData, ParameterError
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
 
 __all__ = [
     "create_ibkr_tick_db",
@@ -37,8 +87,13 @@ __all__ = [
     "download_market_data_file",
 ]
 
-def create_ibkr_tick_db(code, universes=None, sids=None, fields=None,
-                        primary_exchange=False):
+def create_ibkr_tick_db(
+    code: str,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None,
+    primary_exchange: bool = False
+    ) -> dict[str, str]:
     """
     Create a new database for collecting real-time tick data from Interactive
     Brokers.
@@ -69,6 +124,12 @@ def create_ibkr_tick_db(code, universes=None, sids=None, fields=None,
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * IBKR Real-time Data: https://qrok.it/dl/qr/realtime-ibkr
+
     Examples
     --------
     Create a database for collecting real-time trades and volume for US stocks:
@@ -78,7 +139,7 @@ def create_ibkr_tick_db(code, universes=None, sids=None, fields=None,
 
     Create a database for collecting trades and quotes for a universe of futures:
 
-    >>> create_ibkr_tick_db("globex-fut-taq", universes="globex-fut",
+    >>> create_ibkr_tick_db("cme-fut-taq", universes="cme-fut",
                             fields=["LastPrice", "Volume", "BidPrice", "AskPrice", "BidSize", "AskSize"])
     """
     params = {}
@@ -101,7 +162,12 @@ def create_ibkr_tick_db(code, universes=None, sids=None, fields=None,
 def _cli_create_ibkr_tick_db(*args, **kwargs):
     return json_to_cli(create_ibkr_tick_db, *args, **kwargs)
 
-def create_polygon_tick_db(code, universes=None, sids=None, fields=None):
+def create_polygon_tick_db(
+    code: str,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Create a new database for collecting real-time tick data from Polygon.
 
@@ -128,6 +194,12 @@ def create_polygon_tick_db(code, universes=None, sids=None, fields=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Polygon.io Real-time Data: https://qrok.it/dl/qr/realtime-polygon
+
     Examples
     --------
     Create a database for collecting real-time trade prices and sizes for US stocks:
@@ -153,7 +225,12 @@ def create_polygon_tick_db(code, universes=None, sids=None, fields=None):
 def _cli_create_polygon_tick_db(*args, **kwargs):
     return json_to_cli(create_polygon_tick_db, *args, **kwargs)
 
-def create_alpaca_tick_db(code, universes=None, sids=None, fields=None):
+def create_alpaca_tick_db(
+    code: str,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Create a new database for collecting real-time tick data from Alpaca.
 
@@ -180,6 +257,12 @@ def create_alpaca_tick_db(code, universes=None, sids=None, fields=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Alpaca Real-time Data: https://qrok.it/dl/qr/realtime-alpaca
+
     Examples
     --------
     Create a database for collecting real-time trade prices and sizes for US stocks:
@@ -205,7 +288,12 @@ def create_alpaca_tick_db(code, universes=None, sids=None, fields=None):
 def _cli_create_alpaca_tick_db(*args, **kwargs):
     return json_to_cli(create_alpaca_tick_db, *args, **kwargs)
 
-def create_agg_db(code, tick_db_code, bar_size, fields=None):
+def create_agg_db(
+    code: str,
+    tick_db_code: str,
+    bar_size: str,
+    fields: dict[str, list[str]] = None
+    ) -> dict[str, str]:
     """
     Create an aggregate database from a tick database.
 
@@ -236,6 +324,12 @@ def create_agg_db(code, tick_db_code, bar_size, fields=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Aggregate Databases: https://qrok.it/dl/qr/realtime-agg
+
     Examples
     --------
     Create an aggregate database of 1 minute bars consisting of OHLC trades and volume,
@@ -251,7 +345,7 @@ def create_agg_db(code, tick_db_code, bar_size, fields=None):
     the mean bid size and ask size, from a tick database of futures trades and
     quotes, resulting in fields called BidPriceClose, AskPriceClose, BidSizeMean, and AskSizeMean:
 
-    >>> create_agg_db("globex-fut-taq-1sec", tick_db_code="globex-fut-taq",
+    >>> create_agg_db("cme-fut-taq-1sec", tick_db_code="cme-fut-taq",
                       bar_size="1s",
                       fields={"BidPrice":["Close"],
                               "AskPrice": ["Close"],
@@ -284,7 +378,7 @@ def _cli_create_agg_db(*args, **kwargs):
         kwargs["fields"] = dict_strs_to_dict(*fields)
     return json_to_cli(create_agg_db, *args, **kwargs)
 
-def get_db_config(code):
+def get_db_config(code: str) -> dict[str, str]:
     """
     Return the configuration for a tick database or aggregate database.
 
@@ -298,6 +392,11 @@ def get_db_config(code):
     dict
         config
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
     """
     response = houston.get("/realtime/databases/{0}".format(code))
     houston.raise_for_status_with_json(response)
@@ -306,7 +405,11 @@ def get_db_config(code):
 def _cli_get_db_config(*args, **kwargs):
     return json_to_cli(get_db_config, *args, **kwargs)
 
-def drop_db(code, confirm_by_typing_db_code_again=None, cascade=False):
+def drop_db(
+    code: str,
+    confirm_by_typing_db_code_again: str = None,
+    cascade: bool = False
+    ) -> dict[str, str]:
     """
     Delete a tick database or aggregate database.
 
@@ -337,6 +440,12 @@ def drop_db(code, confirm_by_typing_db_code_again=None, cascade=False):
     See Also
     --------
     drop_ticks : Delete ticks from a tick database.
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
     """
     params = {"confirm_by_typing_db_code_again": confirm_by_typing_db_code_again}
     if cascade:
@@ -348,7 +457,10 @@ def drop_db(code, confirm_by_typing_db_code_again=None, cascade=False):
 def _cli_drop_db(*args, **kwargs):
     return json_to_cli(drop_db, *args, **kwargs)
 
-def drop_ticks(code, older_than=None):
+def drop_ticks(
+    code: str,
+    older_than: str = None
+    ) -> dict[str, str]:
     """
     Delete ticks from a tick database. Does not delete any aggregate
     database records.
@@ -381,6 +493,12 @@ def drop_ticks(code, older_than=None):
     --------
     drop_db : Delete a tick database or aggregate database.
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Database Size: https://qrok.it/dl/qr/realtime-dbsize
+
     Examples
     --------
     Delete ticks older than 7 days in a database called 'usa-tech-stk-tick' (no
@@ -396,7 +514,7 @@ def drop_ticks(code, older_than=None):
 def _cli_drop_ticks(*args, **kwargs):
     return json_to_cli(drop_ticks, *args, **kwargs)
 
-def list_databases():
+def list_databases() -> dict[str, list[str]]:
     """
     List tick databases and associated aggregate databases.
 
@@ -405,6 +523,11 @@ def list_databases():
     dict
         dict of {tick_db: [agg_dbs]}
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
     """
     response = houston.get("/realtime/databases")
     houston.raise_for_status_with_json(response)
@@ -413,8 +536,15 @@ def list_databases():
 def _cli_list_databases(*args, **kwargs):
     return json_to_cli(list_databases, *args, **kwargs)
 
-def collect_market_data(codes, sids=None, universes=None, fields=None, until=None,
-                        snapshot=False, wait=False):
+def collect_market_data(
+    codes: Union[list[str], str],
+    sids: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None,
+    until: str = None,
+    snapshot: bool = False,
+    wait: bool = False
+    ) -> dict[str, str]:
     """
     Collect real-time market data and save it to a tick database.
 
@@ -460,6 +590,12 @@ def collect_market_data(codes, sids=None, universes=None, fields=None, until=Non
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
+
     Examples
     --------
     Collect market data for all securities in a tick database called 'japan-banks-trades':
@@ -500,7 +636,7 @@ def collect_market_data(codes, sids=None, universes=None, fields=None, until=Non
 def _cli_collect_market_data(*args, **kwargs):
     return json_to_cli(collect_market_data, *args, **kwargs)
 
-def get_active_collections(detail=False):
+def get_active_collections(detail: bool = False) -> dict[str, dict[str, int]]:
     """
     Return the number of tickers currently being collected, by vendor and
     database.
@@ -516,6 +652,11 @@ def get_active_collections(detail=False):
     dict
         subscribed tickers by vendor and database
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
     """
     params = {}
     if detail:
@@ -528,7 +669,12 @@ def get_active_collections(detail=False):
 def _cli_get_active_collections(*args, **kwargs):
     return json_to_cli(get_active_collections, *args, **kwargs)
 
-def cancel_market_data(codes=None, sids=None, universes=None, cancel_all=False):
+def cancel_market_data(
+    codes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    cancel_all: bool = False
+    ) -> dict[str, dict[str, int]]:
     """
     Cancel market data collection.
 
@@ -551,11 +697,17 @@ def cancel_market_data(codes=None, sids=None, universes=None, cancel_all=False):
     dict
         subscribed tickers by vendor and database, after cancellation
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Data: https://qrok.it/dl/qr/realtime
+
     Examples
     --------
-    Cancel market data collection for a tick database called 'globex-fut-taq':
+    Cancel market data collection for a tick database called 'cme-fut-taq':
 
-    >>> cancel_market_data("globex-fut-taq")
+    >>> cancel_market_data("cme-fut-taq")
 
     Cancel all market data collection:
 
@@ -578,11 +730,18 @@ def cancel_market_data(codes=None, sids=None, universes=None, cancel_all=False):
 def _cli_cancel_market_data(*args, **kwargs):
     return json_to_cli(cancel_market_data, *args, **kwargs)
 
-def download_market_data_file(code, filepath_or_buffer=None, output="csv",
-                              start_date=None, end_date=None,
-                              universes=None, sids=None,
-                              exclude_universes=None, exclude_sids=None,
-                              fields=None):
+def download_market_data_file(
+    code: str,
+    filepath_or_buffer: FilepathOrBuffer = None,
+    output: Literal["csv", "json"] = "csv",
+    start_date: str = None,
+    end_date: str = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Query market data from a tick database or aggregate database and download to file.
 
@@ -629,14 +788,21 @@ def download_market_data_file(code, filepath_or_buffer=None, output="csv",
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Real-time Market Data File: https://qrok.it/dl/qr/realtime
+    * get_prices: https://qrok.it/dl/qr/prices
+
     Examples
     --------
     Download a CSV of futures market data since 08:00 AM Chicago time:
 
-    >>> download_market_data_file("globex-fut-taq",
+    >>> download_market_data_file("cme-fut-taq",
                                  start_date="08:00:00 America/Chicago",
-                                 filepath_or_buffer="globex_taq.csv")
-    >>> market_data = pd.read_csv("globex_taq.csv", parse_dates=["Date"])
+                                 filepath_or_buffer="cme_taq.csv")
+    >>> market_data = pd.read_csv("cme_taq.csv", parse_dates=["Date"])
 
     See Also
     --------
