@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket LLC - All Rights Reserved
+# Copyright 2017-2023 QuantRocket LLC - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,7 +11,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""QuantRocket utility functions.
 
+Functions
+---------
+segmented_date_range
+    Split a date range into smaller segments.
+"""
 from .dt import segmented_date_range
 
 __all__ = ["segmented_date_range"]
\ No newline at end of file
