@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket LLC - All Rights Reserved
+# Copyright 2017-2023 QuantRocket LLC - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-def segmented_date_range(start_date, end_date, segment="A"):
+def segmented_date_range(
+    start_date: str,
+    end_date: str,
+    segment: str = "A"
+    ) -> list[tuple[str, str]]:
     """
     Split a date range into smaller segments.
 
