@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,17 +11,24 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for checking the QuantRocket version number.
 
-import argparse
+Functions
+---------
+get_version
+    Show the QuantRocket version number.
+"""
 from quantrocket import __version__
+from quantrocket.utils._typing import Union
 from quantrocket.houston import houston
-from quantrocket.cli.utils.output import json_to_cli
+from quantrocket._cli.utils.output import json_to_cli
 
 __all__ = [
     "get_version",
 ]
 
-def get_version(detail=False):
+def get_version(detail: bool = False) -> Union[str, dict[str, str]]:
     """
     Show the QuantRocket version number.
 
