@@ -1,4 +1,4 @@
-# Copyright 2017-2022 QuantRocket - All Rights Reserved
+# Copyright 2017-2023 QuantRocket - All Rights Reserved
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,16 +11,90 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""
+Functions for backtesting and trading with Zipline, and managing Zipline
+bundles.
 
+Functions
+---------
+create_usstock_bundle
+    Create a Zipline bundle for US stocks.
+
+create_sharadar_bundle
+    Create a Zipline bundle of daily data for Sharadar stocks and/or ETFs.
+
+create_bundle_from_db
+    Create a Zipline bundle from a history database or real-time aggregate
+    database.
+
+ingest_bundle
+    Ingest data into a previously defined bundle.
+
+list_bundles
+    List available data bundles and whether data has been
+    ingested into them.
+
+get_bundle_config
+    Return the configuration of a bundle.
+
+drop_bundle
+    Delete a bundle.
+
+get_default_bundle
+    Return the current default bundle, if any.
+
+set_default_bundle
+    Set the default bundle to use for backtesting and trading.
+
+download_bundle_file
+    Query minute or daily data from a Zipline bundle and download to a CSV file.
+
+backtest
+    Backtest a Zipline strategy and write the test results to a CSV file.
+
+scan_parameters
+    Run a parameter scan for a Zipline strategy.
+
+trade
+    Trade a Zipline strategy.
+
+list_active_strategies
+    List actively trading Zipline strategies.
+
+cancel_strategies
+    Cancel actively trading strategies.
+
+Classes
+-------
+ZiplineBacktestResult
+    Convenience class for parsing a CSV result file from a Zipline backtest
+    into a variety of useful DataFrames, which can be passed to pyfolio or
+    inspected by the user.
+
+Notes
+-----
+Usage Guide:
+
+* Zipline: https://qrok.it/dl/qr/zipline
+"""
 import sys
 import six
 import requests
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import pandas as pd
+from quantrocket.utils._typing import (
+    FilepathOrBuffer,
+    Union,
+    Any,
+    Literal
+)
 from quantrocket.houston import houston
 from quantrocket.exceptions import NoHistoricalData
-from quantrocket.cli.utils.output import json_to_cli
-from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
-from quantrocket.utils.warn import deprecated_replaced_by
+from quantrocket._cli.utils.output import json_to_cli
+from quantrocket._cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket._cli.utils.parse import dict_strs_to_dict, dict_to_dict_strs
+from quantrocket.utils._warn import deprecated_replaced_by
 
 __all__ = [
     "create_usstock_bundle",
@@ -35,14 +109,19 @@ __all__ = [
     "download_bundle_file",
     "backtest",
     "scan_parameters",
-    "create_tearsheet",
     "trade",
     "list_active_strategies",
     "cancel_strategies",
     "ZiplineBacktestResult",
 ]
 
-def create_usstock_bundle(code, sids=None, universes=None, free=False, data_frequency=None):
+def create_usstock_bundle(
+    code: str,
+    sids: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None,
+    free: bool = False,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None
+    ) -> dict[str, str]:
     """
     Create a Zipline bundle for US stocks.
 
@@ -73,6 +152,12 @@ def create_usstock_bundle(code, sids=None, universes=None, free=False, data_freq
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * US Stock bundle: https://qrok.it/dl/qr/zipline-usstock
+
     Examples
     --------
     Create a minute data bundle for all US stocks:
@@ -110,7 +195,13 @@ def create_usstock_bundle(code, sids=None, universes=None, free=False, data_freq
 def _cli_create_usstock_bundle(*args, **kwargs):
     return json_to_cli(create_usstock_bundle, *args, **kwargs)
 
-def create_sharadar_bundle(code, sec_types=None, free=False):
+def create_sharadar_bundle(
+    code: str,
+    sec_types: Union[list[
+        Literal["STK", "ETF"]],
+        Literal["STK", "ETF"]] = None,
+    free: bool = False
+    ) -> dict[str, str]:
     """
     Create a Zipline bundle of daily data for Sharadar stocks and/or ETFs.
 
@@ -134,6 +225,12 @@ def create_sharadar_bundle(code, sec_types=None, free=False):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Sharadar bundle: https://qrok.it/dl/qr/zipline-sharadar
+
     Examples
     --------
     Create a bundle for all Sharadar stocks and ETFs:
@@ -163,11 +260,18 @@ def create_sharadar_bundle(code, sec_types=None, free=False):
 def _cli_create_sharadar_bundle(*args, **kwargs):
     return json_to_cli(create_sharadar_bundle, *args, **kwargs)
 
-def create_bundle_from_db(code, from_db, calendar,
-                         start_date=None, end_date=None,
-                         universes=None, sids=None,
-                        exclude_universes=None, exclude_sids=None,
-                        fields=None):
+def create_bundle_from_db(
+    code: str,
+    from_db: Union[str, list[str]],
+    calendar: str,
+    start_date: str = None,
+    end_date: str = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    fields: dict[str, str] = None
+    ) -> dict[str, str]:
     """
     Create a Zipline bundle from a history database or real-time aggregate
     database.
@@ -220,6 +324,12 @@ def create_bundle_from_db(code, from_db, calendar,
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * History db bundle: https://qrok.it/dl/qr/zipline-fromdb
+
     Examples
     --------
     Create a bundle from a history database called "es-fut-1min" and name
@@ -276,7 +386,11 @@ def _cli_create_bundle_from_db(*args, **kwargs):
         kwargs["fields"] = dict_strs_to_dict(*fields)
     return json_to_cli(create_bundle_from_db, *args, **kwargs)
 
-def ingest_bundle(code, sids=None, universes=None):
+def ingest_bundle(
+    code: str,
+    sids: Union[list[str], str] = None,
+    universes: Union[list[str], str] = None
+    ) -> dict[str, str]:
     """
     Ingest data into a previously defined bundle.
 
@@ -296,6 +410,12 @@ def ingest_bundle(code, sids=None, universes=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
+
     Examples
     --------
     Ingest data into a bundle called usstock-1min:
@@ -316,7 +436,7 @@ def ingest_bundle(code, sids=None, universes=None):
 def _cli_ingest_bundle(*args, **kwargs):
     return json_to_cli(ingest_bundle, *args, **kwargs)
 
-def list_bundles():
+def list_bundles() -> dict[str, bool]:
     """
     List available data bundles and whether data has been
     ingested into them.
@@ -326,6 +446,12 @@ def list_bundles():
     dict
         data bundles and whether they have data (True indicates
         data, False indicates config only)
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
     """
     response = houston.get("/zipline/bundles")
 
@@ -335,7 +461,7 @@ def list_bundles():
 def _cli_list_bundles(*args, **kwargs):
     return json_to_cli(list_bundles, *args, **kwargs)
 
-def get_bundle_config(code):
+def get_bundle_config(code: str) -> dict[str, str]:
     """
     Return the configuration of a bundle.
 
@@ -348,6 +474,12 @@ def get_bundle_config(code):
     -------
     dict
         config
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
     """
     response = houston.get(f"/zipline/bundles/config/{code}")
 
@@ -357,7 +489,10 @@ def get_bundle_config(code):
 def _cli_get_bundle_config(*args, **kwargs):
     return json_to_cli(get_bundle_config, *args, **kwargs)
 
-def drop_bundle(code, confirm_by_typing_bundle_code_again=None):
+def drop_bundle(
+    code: str,
+    confirm_by_typing_bundle_code_again: str = None
+    ) -> dict[str, str]:
     """
     Delete a bundle.
 
@@ -375,6 +510,12 @@ def drop_bundle(code, confirm_by_typing_bundle_code_again=None):
     dict
         status message
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
+
     Examples
     --------
     Delete a bundle called 'es-fut-1min':
@@ -393,7 +534,7 @@ def drop_bundle(code, confirm_by_typing_bundle_code_again=None):
 def _cli_drop_bundle(*args, **kwargs):
     return json_to_cli(drop_bundle, *args, **kwargs)
 
-def get_default_bundle():
+def get_default_bundle() -> dict[str, str]:
     """
     Return the current default bundle, if any.
 
@@ -401,6 +542,12 @@ def get_default_bundle():
     -------
     dict
         default bundle
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
     """
     response = houston.get("/zipline/config")
     houston.raise_for_status_with_json(response)
@@ -409,7 +556,7 @@ def get_default_bundle():
         return {}
     return response.json()
 
-def set_default_bundle(bundle):
+def set_default_bundle(bundle: str) -> dict[str, str]:
     """
     Set the default bundle to use for backtesting and trading.
 
@@ -426,6 +573,12 @@ def set_default_bundle(bundle):
     -------
     dict
         status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Data bundles: https://qrok.it/dl/qr/zipline-bundles
     """
     data = {
         "default_bundle": bundle
@@ -440,12 +593,19 @@ def _cli_get_or_set_default_bundle(bundle=None, *args, **kwargs):
     else:
         return json_to_cli(get_default_bundle, *args, **kwargs)
 
-def download_bundle_file(code, filepath_or_buffer=None,
-                         start_date=None, end_date=None,
-                         data_frequency=None,
-                         universes=None, sids=None,
-                         exclude_universes=None, exclude_sids=None,
-                         times=None, fields=None):
+def download_bundle_file(
+    code: str,
+    filepath_or_buffer: FilepathOrBuffer = None,
+    start_date: str = None,
+    end_date: str = None,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None,
+    universes: Union[list[str], str] = None,
+    sids: Union[list[str], str] = None,
+    exclude_universes: Union[list[str], str] = None,
+    exclude_sids: Union[list[str], str] = None,
+    times: Union[list[str], str] = None,
+    fields: Union[list[str], str] = None
+    ) -> None:
     """
     Query minute or daily data from a Zipline bundle and download to a CSV file.
 
@@ -492,6 +652,17 @@ def download_bundle_file(code, filepath_or_buffer=None,
     -------
     None
 
+    See Also
+    --------
+    quantrocket.get_prices : load prices into a DataFrame
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Query bundle: https://qrok.it/dl/qr/zipline-query-bundle
+    * get_prices: https://qrok.it/dl/qr/prices
+
     Examples
     --------
     Load minute prices into pandas:
@@ -502,10 +673,6 @@ def download_bundle_file(code, filepath_or_buffer=None,
     Isolate fields with .loc:
 
     >>> closes = prices.loc["Close"]
-
-    See Also
-    --------
-    quantrocket.get_prices : load prices into a DataFrame
     """
     params = {}
     if start_date:
@@ -551,9 +718,17 @@ def download_minute_file(*args, **kwargs):
     kwargs["data_frequency"] = "minute"
     return download_bundle_file(*args, **kwargs)
 
-def backtest(strategy, data_frequency=None, capital_base=None, bundle=None,
-             start_date=None, end_date=None, progress=None, params=None,
-             filepath_or_buffer=None):
+def backtest(
+    strategy: str,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None,
+    capital_base: float = None,
+    bundle: str = None,
+    start_date: str = None,
+    end_date: str = None,
+    progress: str = None,
+    params: dict[str, Any] = None,
+    filepath_or_buffer: FilepathOrBuffer = None
+    ) -> None:
     """
     Backtest a Zipline strategy and write the test results to a CSV file.
 
@@ -593,13 +768,19 @@ def backtest(strategy, data_frequency=None, capital_base=None, bundle=None,
         in the algo file) to modify on the fly before backtesting (pass as
         {param:value}).
 
-    filepath_or_buffer : str, optional
+    filepath_or_buffer : str or file-like object, optional
         the location to write the output file (omit to write to stdout)
 
     Returns
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline backtesting: https://qrok.it/dl/qr/zipline-backtest
+
     Examples
     --------
     Run a backtest defined in momentum-pipeline.py and save to CSV,
@@ -644,11 +825,22 @@ def _cli_backtest(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(backtest, *args, **kwargs)
 
-def scan_parameters(strategy,  data_frequency=None, capital_base=None, bundle=None,
-                    start_date=None, end_date=None,
-                    param1=None, vals1=None, param2=None, vals2=None,
-                    progress=None, params=None, num_workers=None,
-                    filepath_or_buffer=None):
+def scan_parameters(
+    strategy: str,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None,
+    capital_base: float = None,
+    bundle: str = None,
+    start_date: str = None,
+    end_date: str = None,
+    param1: str = None,
+    vals1: list[Any] = None,
+    param2: str = None,
+    vals2: list[Any] = None,
+    progress: str = None,
+    params: dict[str, Any] = None,
+    num_workers: int = None,
+    filepath_or_buffer: FilepathOrBuffer = None
+    ) -> None:
     """
     Run a parameter scan for a Zipline strategy.
 
@@ -719,6 +911,12 @@ def scan_parameters(strategy,  data_frequency=None, capital_base=None, bundle=No
     -------
     None
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline parameter scans: https://qrok.it/dl/qr/zipline-paramscan
+
     Examples
     --------
     Run a parameter scan for a moving average strategy called dma, then
@@ -790,7 +988,10 @@ def _cli_scan_parameters(*args, **kwargs):
         kwargs["params"] = dict_strs_to_dict(*params)
     return json_to_cli(scan_parameters, *args, **kwargs)
 
-def create_tearsheet(infilepath_or_buffer, outfilepath_or_buffer=None):
+def create_tearsheet(
+    infilepath_or_buffer: FilepathOrBuffer,
+    outfilepath_or_buffer: FilepathOrBuffer = None
+    ) -> None:
     """
     Create a pyfolio PDF tear sheet from a Zipline backtest result.
 
@@ -831,7 +1032,13 @@ def create_tearsheet(infilepath_or_buffer, outfilepath_or_buffer=None):
 def _cli_create_tearsheet(*args, **kwargs):
     return json_to_cli(create_tearsheet, *args, **kwargs)
 
-def trade(strategy, bundle=None, account=None, data_frequency=None, dry_run=False):
+def trade(
+    strategy: str,
+    bundle: str = None,
+    account: str = None,
+    data_frequency: Literal["daily", "minute", "d", "m"] = None,
+    dry_run: bool = False
+    ) -> dict[str, str]:
     """
     Trade a Zipline strategy.
 
@@ -862,7 +1069,14 @@ def trade(strategy, bundle=None, account=None, data_frequency=None, dry_run=Fals
 
     Returns
     -------
-    None
+    dict
+        status message
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline live trading: https://qrok.it/dl/qr/zipline-trade
 
     Examples
     --------
@@ -888,7 +1102,7 @@ def trade(strategy, bundle=None, account=None, data_frequency=None, dry_run=Fals
 def _cli_trade(*args, **kwargs):
     return json_to_cli(trade, *args, **kwargs)
 
-def list_active_strategies():
+def list_active_strategies() -> dict[str, list[str]]:
     """
     List actively trading Zipline strategies.
 
@@ -896,6 +1110,12 @@ def list_active_strategies():
     -------
     dict
         dict of account: strategies
+
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline live trading: https://qrok.it/dl/qr/zipline-trade
     """
     response = houston.get("/zipline/trade")
 
@@ -905,7 +1125,11 @@ def list_active_strategies():
 def _cli_list_active_strategies(*args, **kwargs):
     return json_to_cli(list_active_strategies, *args, **kwargs)
 
-def cancel_strategies(strategies=None, accounts=None, cancel_all=False):
+def cancel_strategies(
+    strategies: Union[list[str], str] = None,
+    accounts: Union[list[str], str] = None,
+    cancel_all: bool = False
+    ) -> dict[str, list[str]]:
     """
     Cancel actively trading strategies.
 
@@ -925,6 +1149,12 @@ def cancel_strategies(strategies=None, accounts=None, cancel_all=False):
     dict
         dict of actively trading strategies after canceling
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline live trading: https://qrok.it/dl/qr/zipline-trade
+
     Examples
     --------
     Cancel a single strategy:
@@ -957,6 +1187,12 @@ class ZiplineBacktestResult(object):
     into a variety of useful DataFrames, which can be passed to pyfolio or
     inspected by the user.
 
+    Notes
+    -----
+    Usage Guide:
+
+    * Zipline backtesting: https://qrok.it/dl/qr/zipline-backtest
+
     Examples
     --------
     Run a Zipline backtest and parse the CSV results:
@@ -988,15 +1224,53 @@ class ZiplineBacktestResult(object):
     """
 
     def __init__(self):
-        self.returns = None
-        self.positions = None
-        self.transactions = None
-        self.benchmark_returns = None
-        self.perf = None
+        self.returns: 'pd.Series[float]' = None
+        self.positions: 'pd.DataFrame' = None
+        self.transactions: 'pd.DataFrame' = None
+        self.benchmark_returns: 'pd.Series[float]' = None
+        self.perf: 'pd.DataFrame' = None
 
     @classmethod
-    def from_csv(cls, filepath_or_buffer):
+    def from_csv(cls, filepath_or_buffer: FilepathOrBuffer) -> 'ZiplineBacktestResult':
+        """
+        Parse a CSV result file from a Zipline backtest into a variety of useful
+        DataFrames, which can be passed to pyfolio or inspected by the user.
 
+        Notes
+        -----
+        Usage Guide:
+
+        * Zipline backtesting: https://qrok.it/dl/qr/zipline-backtest
+
+        Examples
+        --------
+        Run a Zipline backtest and parse the CSV results:
+
+        >>> f = io.StringIO()
+        >>> backtest("momentum_pipeline.py",
+                    bundle="etf-sampler-1d",
+                    start="2015-02-04",
+                    end="2015-12-31",
+                    filepath_or_buffer=f)
+        >>> zipline_result = ZiplineBacktestResult.from_csv(f)
+
+        The ZiplineBacktestResult object contains returns, positions, transactions,
+        benchmark_returns, and the performance DataFrame.
+
+        >>> print(zipline_result.returns.head())
+        >>> print(zipline_result.positions.head())
+        >>> print(zipline_result.transactions.head())
+        >>> print(zipline_result.benchmark_returns.head())
+        >>> print(zipline_result.perf.head())
+
+        The outputs are ready to be passed to pyfolio:
+
+        >>> pf.create_full_tear_sheet(
+                zipline_result.returns,
+                positions=zipline_result.positions,
+                transactions=zipline_result.transactions,
+                benchmark_rets=zipline_result.benchmark_returns)
+        """
         # Import pandas lazily since it can take a moment to import
         try:
             import pandas as pd
@@ -1060,3 +1334,5 @@ class ZiplineBacktestResult(object):
         zipline_result.perf = perf.apply(pd.to_numeric, errors='ignore')
 
         return zipline_result
+
+ZiplineBacktestResult.from_csv.__func__.__doc__ = ZiplineBacktestResult.__doc__
